@@ -13,6 +13,7 @@ import (
 	"net/url"
 	"os"
 	"path/filepath"
+	"sort"
 	"strings"
 	"time"
 
@@ -24,6 +25,7 @@ import (
 	"github.com/spf13/cobra"
 	"github.com/spf13/pflag"
 	"github.com/zclconf/go-cty/cty"
+	"golang.org/x/exp/maps"
 	"golang.org/x/mod/semver"
 )
 
@@ -88,6 +90,56 @@ Atlas is licensed under Apache 2.0 as found in https://github.com/ariga/atlas/bl
 	}
 )
 
+type (
+	// ErrorFormatter implemented by the errors below to
+	// allow them format command output on error.
+	ErrorFormatter interface {
+		FormatError(*cobra.Command)
+	}
+	// FormattedError is an error that format the command output when returned.
+	FormattedError struct {
+		Err    error
+		Prefix string // Prefix to use on error.
+		Silent bool   // Silent errors are not printed.
+	}
+	// AbortError returns a command error that is formatted as "Abort: ..." when
+	// the execution is aborted by the user.
+	AbortError struct {
+		Err error
+	}
+)
+
+func (e *FormattedError) Error() string { return e.Err.Error() }
+
+func (e *FormattedError) FormatError(cmd *cobra.Command) {
+	cmd.SilenceErrors = e.Silent
+	if e.Prefix != "" {
+		cmd.SetErrPrefix(e.Prefix)
+	}
+}
+
+// AbortErrorf is like fmt.Errorf for creating AbortError.
+func AbortErrorf(format string, a ...any) error {
+	return &AbortError{Err: fmt.Errorf(format, a...)}
+}
+
+func (e *AbortError) Error() string { return e.Err.Error() }
+
+func (e *AbortError) FormatError(cmd *cobra.Command) {
+	cmd.SetErrPrefix("Abort:")
+}
+
+// RunE wraps the command cobra.Command.RunE function with additional postrun logic.
+func RunE(f func(*cobra.Command, []string) error) func(*cobra.Command, []string) error {
+	return func(cmd *cobra.Command, args []string) error {
+		err := f(cmd, args)
+		if ef, ok := err.(ErrorFormatter); ok {
+			ef.FormatError(cmd)
+		}
+		return err
+	}
+}
+
 func init() {
 	Root.AddCommand(versionCmd)
 	Root.AddCommand(licenseCmd)
@@ -149,14 +201,32 @@ type Vars map[string]cty.Value
 
 // String implements pflag.Value.String.
 func (v Vars) String() string {
-	var b strings.Builder
-	for k := range v {
+	var (
+		b  strings.Builder
+		ks = maps.Keys(v)
+	)
+	sort.Strings(ks)
+	for _, k := range ks {
 		if b.Len() > 0 {
 			b.WriteString(", ")
 		}
 		b.WriteString(k)
 		b.WriteString(":")
-		b.WriteString(v[k].GoString())
+		switch v1 := v[k]; v1.Type() {
+		case cty.String:
+			b.WriteString(v1.AsString())
+		case cty.List(cty.String):
+			b.WriteString("[")
+			for i, v2 := range v1.AsValueSlice() {
+				if i > 0 {
+					b.WriteString(", ")
+				}
+				b.WriteString(v2.AsString())
+			}
+			b.WriteString("]")
+		default:
+			b.WriteString(v1.GoString())
+		}
 	}
 	return "[" + b.String() + "]"
 }
@@ -213,6 +283,7 @@ const (
 	flagAutoApprove    = "auto-approve"
 	flagBaseline       = "baseline"
 	flagConfig         = "config"
+	flagContext        = "context"
 	flagDevURL         = "dev-url"
 	flagDirURL         = "dir"
 	flagDirFormat      = "dir-format"
@@ -233,6 +304,7 @@ const (
 	flagSchemaShort    = "s"
 	flagTo             = "to"
 	flagTxMode         = "tx-mode"
+	flagExecOrder      = "exec-order"
 	flagURL            = "url"
 	flagURLShort       = "u"
 	flagVar            = "var"
@@ -435,16 +507,28 @@ func (c *stateReaderConfig) Exported() (*cmdext.StateReaderConfig, error) {
 }
 
 // readerUseDev reports if any of the URL uses the dev-database.
-func readerUseDev(urls ...string) (bool, error) {
+func readerUseDev(env *Env, urls ...string) (bool, error) {
 	s, err := selectScheme(urls)
 	if err != nil {
 		return false, err
 	}
-	return s == "file" || cmdext.States.HasLoader(s), nil
+	switch {
+	case s == envAttrScheme && env != nil && len(urls) == 1:
+		u, err := env.VarFromURL(urls[0])
+		if err != nil {
+			return false, err
+		}
+		// No circular reference possible with env:// variable.
+		return readerUseDev(env, u)
+	case s == cmdext.SchemaTypeFile, s == cmdext.SchemaTypeAtlas:
+		return true, nil
+	default:
+		return cmdext.States.HasLoader(s), nil
+	}
 }
 
 // stateReader returns a migrate.StateReader that reads the state from the given urls.
-func stateReader(ctx context.Context, config *stateReaderConfig) (*cmdext.StateReadCloser, error) {
+func stateReader(ctx context.Context, env *Env, config *stateReaderConfig) (*cmdext.StateReadCloser, error) {
 	excfg, err := config.Exported()
 	if err != nil {
 		return nil, err
@@ -455,17 +539,37 @@ func stateReader(ctx context.Context, config *stateReaderConfig) (*cmdext.StateR
 	}
 	switch scheme {
 	// "file" scheme is valid for both migration directory and HCL paths.
-	case "file":
+	case cmdext.SchemaTypeFile:
 		switch ext, err := filesExt(excfg.URLs); {
 		case err != nil:
 			return nil, err
-		case ext == extHCL:
+		case ext == cmdext.FileTypeHCL:
 			return cmdext.StateReaderHCL(ctx, excfg)
-		case ext == extSQL:
+		case ext == cmdext.FileTypeSQL:
 			return cmdext.StateReaderSQL(ctx, excfg)
 		default:
 			panic("unreachable") // checked by filesExt.
 		}
+	// "atlas" scheme represents an Atlas Cloud schema.
+	case cmdext.SchemaTypeAtlas:
+		return cmdext.StateReaderAtlas(ctx, excfg)
+	// "env" scheme represents an attribute defined
+	// on the selected environment.
+	case envAttrScheme:
+		switch {
+		case GlobalFlags.SelectedEnv == "":
+			return nil, errors.New("cannot use env:// variables without selecting an environment")
+		case len(config.urls) != 1:
+			return nil, errors.New("cannot use multiple env:// variables in a single flag")
+		default:
+			u, err := env.VarFromURL(config.urls[0])
+			if err != nil {
+				return nil, err
+			}
+			cfg := *config
+			cfg.urls = []string{u}
+			return stateReader(ctx, env, &cfg)
+		}
 	default:
 		// In case there is an external state-loader registered with this scheme.
 		if l, ok := cmdext.States.Loader(scheme); ok {
@@ -498,16 +602,11 @@ func stateReader(ctx context.Context, config *stateReaderConfig) (*cmdext.StateR
 	}
 }
 
-const (
-	extHCL = ".hcl"
-	extSQL = ".sql"
-)
-
 func filesExt(urls []*url.URL) (string, error) {
 	var path, ext string
 	set := func(curr string) error {
 		switch e := filepath.Ext(curr); {
-		case e != extHCL && e != extSQL:
+		case e != cmdext.FileTypeHCL && e != cmdext.FileTypeSQL:
 			return fmt.Errorf("unknown schema file: %q", curr)
 		case ext != "" && ext != e:
 			return fmt.Errorf("ambiguous schema: both SQL and HCL files found: %q, %q", path, curr)
@@ -529,7 +628,7 @@ func filesExt(urls []*url.URL) (string, error) {
 			for _, f := range files {
 				switch filepath.Ext(f.Name()) {
 				// Ignore unknown extensions in case we read directories.
-				case extHCL, extSQL:
+				case cmdext.FileTypeHCL, cmdext.FileTypeSQL:
 					if err := set(f.Name()); err != nil {
 						return "", err
 					}
