@@ -8,6 +8,7 @@ import (
 	"bytes"
 	"context"
 	"database/sql"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
@@ -32,9 +33,6 @@ import (
 	"ariga.io/atlas/sql/sqltool"
 
 	"github.com/google/uuid"
-	"github.com/hashicorp/hcl/v2"
-	"github.com/hashicorp/hcl/v2/hclparse"
-	"github.com/hashicorp/hcl/v2/hclsyntax"
 	"github.com/spf13/cobra"
 )
 
@@ -59,16 +57,29 @@ type migrateApplyFlags struct {
 	allowDirty      bool   // allow working on a database that already has resources
 	baselineVersion string // apply with this version as baseline
 	txMode          string // (none, file, all)
+	execOrder       string // (linear, linear-skip, non-linear)
+	context         string // Run context. See cloudapi.DeployContextInput.
 }
 
-func (f *migrateApplyFlags) migrateOptions() (opts []migrate.ExecutorOption) {
+func (f *migrateApplyFlags) migrateOptions() ([]migrate.ExecutorOption, error) {
+	var opts []migrate.ExecutorOption
 	if f.allowDirty {
 		opts = append(opts, migrate.WithAllowDirty(true))
 	}
 	if v := f.baselineVersion; v != "" {
 		opts = append(opts, migrate.WithBaselineVersion(v))
 	}
-	return
+	if v := f.execOrder; v != "" && v != execOrderLinear {
+		switch v {
+		case execOrderLinearSkip:
+			opts = append(opts, migrate.WithExecOrder(migrate.ExecOrderLinearSkip))
+		case execOrderNonLinear:
+			opts = append(opts, migrate.WithExecOrder(migrate.ExecOrderNonLinear))
+		default:
+			return nil, fmt.Errorf("unknown execution order: %q", v)
+		}
+	}
+	return opts, nil
 }
 
 func migrateApplyCmd() *cobra.Command {
@@ -90,7 +101,7 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
   atlas migrate apply --env dev 1
   atlas migrate apply --dry-run --env dev 1`,
 			Args: cobra.MaximumNArgs(1),
-			RunE: func(cmd *cobra.Command, args []string) (cmdErr error) {
+			RunE: RunE(func(cmd *cobra.Command, args []string) (cmdErr error) {
 				switch {
 				case GlobalFlags.SelectedEnv == "":
 					if err := migrateFlagsFromConfig(cmd); err != nil {
@@ -98,14 +109,15 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 					}
 					return migrateApplyRun(cmd, args, flags, &MigrateReport{}) // nop reporter
 				default:
-					project, envs, err := EnvByName(cmd, GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
+					project, envs, err := EnvByName(cmd, GlobalFlags.SelectedEnv, GlobalFlags.Vars)
 					if err != nil {
 						return err
 					}
-					var (
-						hasRemote bool
-						set       = NewReportProvider(cmd.Context(), project, envs)
-					)
+					set, err := NewReportProvider(cmd.Context(), project, envs, &flags)
+					if err != nil {
+						return err
+					}
+					var hasRemote bool
 					defer func() {
 						if hasRemote {
 							set.Flush(cmd, cmdErr)
@@ -119,7 +131,7 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 						return migrateApplyRun(cmd, args, flags, set.ReportFor(flags, env))
 					})
 				}
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -132,6 +144,9 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 	addFlagLockTimeout(cmd.Flags(), &flags.lockTimeout)
 	cmd.Flags().StringVarP(&flags.baselineVersion, flagBaseline, "", "", "start the first migration after the given baseline version")
 	cmd.Flags().StringVarP(&flags.txMode, flagTxMode, "", txModeFile, "set transaction mode [none, file, all]")
+	cmd.Flags().StringVarP(&flags.execOrder, flagExecOrder, "", execOrderLinear, "set file execution order [linear, linear-skip, non-linear]")
+	cmd.Flags().StringVar(&flags.context, flagContext, "", "describes what triggered this command (e.g., GitHub Action)")
+	cobra.CheckErr(cmd.Flags().MarkHidden(flagContext))
 	cmd.Flags().BoolVarP(&flags.allowDirty, flagAllowDirty, "", false, "allow start working on a non-clean database")
 	cmd.MarkFlagsMutuallyExclusive(flagLog, flagFormat)
 	return cmd
@@ -203,7 +218,11 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags,
 		return err
 	}
 	// Determine pending files.
-	opts := append(flags.migrateOptions(), migrate.WithOperatorVersion(operatorVersion()), migrate.WithLogger(report))
+	opts, err := flags.migrateOptions()
+	if err != nil {
+		return err
+	}
+	opts = append(opts, migrate.WithOperatorVersion(operatorVersion()), migrate.WithLogger(report))
 	ex, err := migrate.NewExecutor(client.Driver, dir, rrw, opts...)
 	if err != nil {
 		return err
@@ -283,7 +302,7 @@ type (
 )
 
 // NewReportProvider returns a new ReporterProvider.
-func NewReportProvider(ctx context.Context, p *Project, envs []*Env) *MigrateReportSet {
+func NewReportProvider(ctx context.Context, p *Project, envs []*Env, flags *migrateApplyFlags) (*MigrateReportSet, error) {
 	c := cloudapi.FromContext(ctx)
 	if p.cfg.Client != nil {
 		c = p.cfg.Client
@@ -296,11 +315,16 @@ func NewReportProvider(ctx context.Context, p *Project, envs []*Env) *MigrateRep
 			Planned:   len(envs),
 		},
 	}
+	if flags.context != "" {
+		if err := json.Unmarshal([]byte(flags.context), &s.Context); err != nil {
+			return nil, fmt.Errorf("invalid --context: %w", err)
+		}
+	}
 	s.Step("Start migration for %d targets", len(envs))
 	for _, e := range envs {
 		s.StepLog(s.RedactedURL(e.URL))
 	}
-	return s
+	return s, nil
 }
 
 // RedactedURL returns the redacted URL of the given environment at index i.
@@ -386,6 +410,7 @@ func (s *MigrateReportSet) Flush(cmd *cobra.Command, cmdErr error) {
 		return
 	// Single migration that was completed.
 	case s.Planned == 1 && len(s.Completed) == 1:
+		s.Completed[0].Context = s.Context
 		link, err = s.client.ReportMigration(cmd.Context(), s.Completed[0])
 	// Single migration that failed to start.
 	case s.Planned == 1 && len(s.Completed) == 0:
@@ -481,7 +506,7 @@ func (r *MigrateReport) Done(cmd *cobra.Command, flags migrateApplyFlags) error
 		Files: func() []cloudapi.DeployedFileInput {
 			files := make([]cloudapi.DeployedFileInput, len(r.log.Applied))
 			for i, f := range r.log.Applied {
-				files[i] = cloudapi.DeployedFileInput{
+				f1 := cloudapi.DeployedFileInput{
 					Name:      f.Name(),
 					Content:   string(f.Bytes()),
 					StartTime: f.Start,
@@ -489,7 +514,25 @@ func (r *MigrateReport) Done(cmd *cobra.Command, flags migrateApplyFlags) error
 					Skipped:   f.Skipped,
 					Applied:   len(f.Applied),
 					Error:     (*cloudapi.StmtErrorInput)(f.Error),
+					Checks:    make([]cloudapi.FileChecksInput, 0, len(f.Checks)),
 				}
+				for _, c := range f.Checks {
+					stmts := make([]cloudapi.CheckStmtInput, 0, len(c.Stmts))
+					for _, s := range c.Stmts {
+						stmts = append(stmts, cloudapi.CheckStmtInput{
+							Stmt:  s.Stmt,
+							Error: s.Error,
+						})
+					}
+					f1.Checks = append(f1.Checks, cloudapi.FileChecksInput{
+						Name:   c.Name,
+						Start:  c.Start,
+						End:    c.End,
+						Checks: stmts,
+						Error:  (*cloudapi.StmtErrorInput)(c.Error),
+					})
+				}
+				files[i] = f1
 			}
 			return files
 		}(),
@@ -545,7 +588,7 @@ func migrateDiffCmd() *cobra.Command {
 by executing its files. It then compares its state to the desired state and create a new migration file containing
 SQL statements for moving from the current to the desired state. The desired state can be another another database,
 an HCL, SQL, or ORM schema. See: https://atlasgo.io/versioned/diff`,
-			Example: `  atlas migrate diff "--dev-url docker://mysql/8/dev" --to "file://schema.hcl"
+			Example: `  atlas migrate diff --dev-url "docker://mysql/8/dev" --to "file://schema.hcl"
   atlas migrate diff --dev-url "docker://postgres/15/dev?search_path=public" --to "file://atlas.hcl" add_users_table
   atlas migrate diff --dev-url "mysql://user:pass@localhost:3306/dev" --to "mysql://user:pass@localhost:3306/dbname"
   atlas migrate diff --env dev --format '{{ sql . "  " }}'`,
@@ -559,13 +602,13 @@ an HCL, SQL, or ORM schema. See: https://atlasgo.io/versioned/diff`,
 				}
 				return checkDir(cmd, flags.dirURL, true)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				env, err := selectEnv(cmd)
 				if err != nil {
 					return err
 				}
 				return migrateDiffRun(cmd, args, flags, env)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -608,6 +651,9 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags, e
 	if err != nil {
 		return err
 	}
+	if err := checkDirRebased(ctx, cmd, dir); err != nil {
+		return err
+	}
 	if flags.edit {
 		dir = &editDir{dir}
 	}
@@ -636,7 +682,7 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags, e
 		return maskNoPlan(cmd, err)
 	}
 	// Get a state reader for the desired state.
-	desired, err := stateReader(ctx, &stateReaderConfig{
+	desired, err := stateReader(ctx, env, &stateReaderConfig{
 		urls:    flags.desiredURLs,
 		dev:     dev,
 		client:  dev,
@@ -751,7 +797,7 @@ This command should be used whenever a manual change in the migration directory
 				}
 				return dirFormatBC(flags.dirFormat, &flags.dirURL)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				dir, err := cmdmigrate.Dir(cmd.Context(), flags.dirURL, false)
 				if err != nil {
 					return err
@@ -761,7 +807,7 @@ This command should be used whenever a manual change in the migration directory
 					return err
 				}
 				return migrate.WriteSumFile(dir, sum)
-			},
+			}),
 		}
 	)
 	addFlagDirURL(cmd.Flags(), &flags.dirURL)
@@ -800,9 +846,9 @@ func migrateImportCmd() *cobra.Command {
 				}
 				return nil
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				return migrateImportRun(cmd, args, flags)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -852,7 +898,7 @@ func migrateImportRun(cmd *cobra.Command, _ []string, flags migrateImportFlags)
 	// Extract the statements for each of the migration files,
 	// add them to a plan to format with the DefaultFormatter.
 	for _, f := range ff {
-		stmts, err := f.StmtDecls()
+		stmts, err := f.StmtDecls() // Not driver aware.
 		if err != nil {
 			return err
 		}
@@ -919,9 +965,9 @@ func migrateLintCmd() *cobra.Command {
 				}
 				return dirFormatBC(flags.dirFormat, &flags.dirURL)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				return migrateLintRun(cmd, args, flags)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -964,9 +1010,9 @@ func migrateNewCmd() *cobra.Command {
 				}
 				return checkDir(cmd, flags.dirURL, true)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				return migrateNewRun(cmd, args, flags)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -1026,9 +1072,9 @@ to be applied. This command is usually used after manually making changes to the
 				}
 				return checkDir(cmd, flags.dirURL, false)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				return migrateSetRun(cmd, args, flags)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -1221,9 +1267,9 @@ func migrateStatusCmd() *cobra.Command {
 				}
 				return checkDir(cmd, flags.dirURL, false)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				return migrateStatusRun(cmd, args, flags)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -1296,9 +1342,9 @@ files are executed on the connected database in order to validate SQL semantics.
 				}
 				return checkDir(cmd, flags.dirURL, false)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				return migrateValidateRun(cmd, args, flags)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -1414,6 +1460,10 @@ const (
 	txModeNone = "none"
 	txModeAll  = "all"
 	txModeFile = "file"
+
+	execOrderLinear     = "linear"
+	execOrderLinearSkip = "linear-skip"
+	execOrderNonLinear  = "non-linear"
 )
 
 // tx handles wrapping migration execution in transactions.
@@ -1585,7 +1635,7 @@ func selectScheme(urls []string) (string, error) {
 			ex := filepath.Ext(u)
 			switch f, err := os.Stat(u); {
 			case err != nil:
-			case f.IsDir(), ex == extSQL, ex == extHCL:
+			case f.IsDir(), ex == cmdext.FileTypeSQL, ex == cmdext.FileTypeHCL:
 				return "", fmt.Errorf("missing scheme. Did you mean file://%s?", u)
 			}
 			return "", errors.New("missing scheme. See: https://atlasgo.io/url")
@@ -1593,38 +1643,13 @@ func selectScheme(urls []string) (string, error) {
 			scheme = current
 		case scheme != current:
 			return "", fmt.Errorf("got mixed --to url schemes: %q and %q, the desired state must be provided from a single kind of source", scheme, current)
-		case current != "file":
+		case current != cmdext.SchemaTypeFile:
 			return "", fmt.Errorf("got multiple --to urls of scheme %q, only multiple 'file://' urls are supported", current)
 		}
 	}
 	return scheme, nil
 }
 
-// mayParse will parse the file in path if it is an HCL file. If the file is an Atlas
-// project file an error is returned.
-func mayParse(p *hclparse.Parser, path string) error {
-	if n := filepath.Base(path); filepath.Ext(n) != extHCL {
-		return nil
-	}
-	switch f, diag := p.ParseHCLFile(path); {
-	case diag.HasErrors():
-		return diag
-	case isProjectFile(f):
-		return fmt.Errorf("cannot parse project file %q as a schema file", path)
-	default:
-		return nil
-	}
-}
-
-func isProjectFile(f *hcl.File) bool {
-	for _, blk := range f.Body.(*hclsyntax.Body).Blocks {
-		if blk.Type == "env" {
-			return true
-		}
-	}
-	return false
-}
-
 func migrateFlagsFromConfig(cmd *cobra.Command) error {
 	env, err := selectEnv(cmd)
 	if err != nil {
@@ -1663,6 +1688,9 @@ func setMigrateEnvFlags(cmd *cobra.Command, env *Env) error {
 		if err := maySetFlag(cmd, flagLockTimeout, env.Migration.LockTimeout); err != nil {
 			return err
 		}
+		if err := maySetFlag(cmd, flagExecOrder, strings.ReplaceAll(strings.ToLower(env.Migration.ExecOrder), "_", "-")); err != nil {
+			return err
+		}
 	case "diff", "checkpoint":
 		if err := maySetFlag(cmd, flagLockTimeout, env.Migration.LockTimeout); err != nil {
 			return err
