@@ -193,8 +193,8 @@ func TestMigrate_Apply(t *testing.T) {
 	require.Contains(t, s, "20220318104614")                           // log to version
 	require.Contains(t, s, "CREATE TABLE tbl (`col` int NOT NULL);")   // logs statement
 	require.NotContains(t, s, "ALTER TABLE `tbl` ADD `col_2` bigint;") // does not execute second file
-	require.Contains(t, s, "1 migrations")                             // logs amount of migrations
-	require.Contains(t, s, "1 sql statements")
+	require.Contains(t, s, "1 migration")                              // logs amount of migrations
+	require.Contains(t, s, "1 sql statement")
 
 	// Transactions will be wrapped per file. If the second file has an error, first still is applied.
 	s, err = runCmd(
@@ -208,8 +208,8 @@ func TestMigrate_Apply(t *testing.T) {
 	require.Contains(t, s, "ALTER TABLE `tbl` ADD `col_2` bigint;")    // does execute first stmt first second file
 	require.Contains(t, s, "ALTER TABLE `tbl` ADD `col_3` bigint;")    // does execute second stmt first second file
 	require.NotContains(t, s, "ALTER TABLE `tbl` ADD `col_4` bigint;") // but not third
-	require.Contains(t, s, "1 migrations ok (1 with errors)")          // logs amount of migrations
-	require.Contains(t, s, "2 sql statements ok (1 with errors)")      // logs amount of statement
+	require.Contains(t, s, "-- 1 migration ok, 1 with errors")         // logs amount of migrations
+	require.Contains(t, s, "-- 2 sql statements ok, 1 with errors")    // logs amount of statement
 	require.Contains(t, s, "near \"asdasd\": syntax error")            // logs error summary
 
 	c, err := sqlclient.Open(ctx, fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test2.db")))
@@ -243,8 +243,8 @@ func TestMigrate_Apply(t *testing.T) {
 	require.Contains(t, s, "ALTER TABLE `tbl` ADD `col_2` bigint;")     // picks up first statement
 	require.Contains(t, s, "ALTER TABLE `tbl` ADD `col_3` bigint;")     // does execute second stmt first second file
 	require.NotContains(t, s, "ALTER TABLE `tbl` ADD `col_4` bigint;")  // but not third
-	require.Contains(t, s, "0 migrations ok (1 with errors)")           // logs amount of migrations
-	require.Contains(t, s, "1 sql statements ok (1 with errors)")       // logs amount of statement
+	require.Contains(t, s, "-- 1 migration with errors")                // logs amount of migrations
+	require.Contains(t, s, "-- 1 sql statement ok, 1 with errors")      // logs amount of statement
 	require.Contains(t, s, "near \"asdasd\": syntax error")             // logs error summary
 
 	// Editing an applied line will raise error.
@@ -533,6 +533,8 @@ env "local" {
 			"-c", "file://"+path,
 			"--env", "local",
 			"--var", fmt.Sprintf("url=sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "tenants.db")),
+			// Inject fake variable to enforce re-evaluation of the data source (skip cache).
+			"--var", fmt.Sprintf("cache=%s", uuid.NewString()),
 		)
 		// Rows should represent real and consistent values.
 		require.EqualError(t, err, "data.sql.tenants: unsupported row type: <nil>")
@@ -544,6 +546,8 @@ env "local" {
 			"-c", "file://"+path,
 			"--env", "local",
 			"--var", fmt.Sprintf("url=sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "tenants.db")),
+			// Inject fake variable to enforce re-evaluation of the data source (skip cache).
+			"--var", fmt.Sprintf("cache=%s", uuid.NewString()),
 		)
 		// Empty list is expanded to zero blocks.
 		require.EqualError(t, err, `env "local" not defined in project file`)
@@ -680,7 +684,7 @@ func TestMigrate_ApplyTxModeDirective(t *testing.T) {
 			"--url", u,
 			"--tx-mode", mode,
 		)
-		require.EqualError(t, err, `sql/migrate: execute: executing statement "INSERT INTO t1 VALUES (1), (1);" from version "20220925094021": UNIQUE constraint failed: t1.a`)
+		require.EqualError(t, err, `sql/migrate: executing statement "INSERT INTO t1 VALUES (1), (1);" from version "20220925094021": UNIQUE constraint failed: t1.a`)
 		db, err := sql.Open("sqlite3", strings.TrimPrefix(u, "sqlite://"))
 		require.NoError(t, err)
 		var n int
@@ -709,6 +713,95 @@ func TestMigrate_ApplyTxModeDirective(t *testing.T) {
 	require.Equal(t, s, `unknown txmode "unknown" found in file directive "20220925094021_second.sql"`)
 }
 
+func TestMigrate_ApplyExecOrder(t *testing.T) {
+	p := t.TempDir()
+	db := fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test.db"))
+	require.NoError(t, os.Mkdir(filepath.Join(p, "migrations"), 0700))
+	dir, err := migrate.NewLocalDir(filepath.Join(p, "migrations"))
+	require.NoError(t, err)
+	write := func(n, b string) {
+		require.NoError(t, dir.WriteFile(n, []byte(b)))
+		hash, err := dir.Checksum()
+		require.NoError(t, err)
+		require.NoError(t, migrate.WriteSumFile(dir, hash))
+	}
+	write("1.sql", "create table t1(c int);")
+	write("3.sql", "create table t3(c int);")
+
+	// First run.
+	s, err := runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://"+dir.Path(),
+		"--url", db,
+		"--format", "{{ len .Applied }}",
+	)
+	require.NoError(t, err)
+	require.Equal(t, "2", s)
+
+	// File was added out of order.
+	write("2.sql", "create table t2(c int);")
+	_, err = runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://"+dir.Path(),
+		"--url", db,
+	)
+	require.EqualError(t, err, "migration file 2.sql was added out of order. See: https://atlasgo.io/versioned/apply#non-linear-error")
+
+	// The "linear" option is the default execution order.
+	_, err = runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://"+dir.Path(),
+		"--url", db,
+		"--exec-order", "linear",
+	)
+	require.EqualError(t, err, "migration file 2.sql was added out of order. See: https://atlasgo.io/versioned/apply#non-linear-error")
+
+	// Keep linear order and skip files that were added out of order.
+	s, err = runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://"+dir.Path(),
+		"--url", db,
+		"--exec-order", "linear-skip",
+	)
+	require.NoError(t, err)
+	require.Equal(t, "No migration files to execute\n", s)
+
+	// Allow non-linear order.
+	s, err = runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://"+dir.Path(),
+		"--url", db,
+		"--exec-order", "non-linear",
+		"--format", "{{ range .Applied }}{{ .Version }}{{ end }}",
+	)
+	require.NoError(t, err)
+	require.Equal(t, "2", s)
+
+	write("2.5.sql", "create table t25(c int);")
+	write("4.sql", "create table t4(c int);")
+	s, err = runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://"+dir.Path(),
+		"--url", db,
+		"--exec-order", "non-linear",
+		"--format", "{{ range .Applied }}{{ println .Version }}{{ end }}",
+	)
+	require.NoError(t, err)
+	require.Equal(t, "2.5\n4\n", s)
+
+	// There are no pending migrations, in all execution orders.
+	for _, o := range []string{"linear", "linear-skip", "non-linear"} {
+		s, err = runCmd(
+			migrateApplyCmd(),
+			"--dir", "file://"+dir.Path(),
+			"--url", db,
+			"--exec-order", o,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "No migration files to execute\n", s)
+	}
+}
+
 func TestMigrate_ApplyBaseline(t *testing.T) {
 	t.Run("FromFlags", func(t *testing.T) {
 		p := t.TempDir()
@@ -802,7 +895,7 @@ func TestMigrate_ApplyCloudReport(t *testing.T) {
 			}
 			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
 			switch {
-			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "bot"):
+			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "me"):
 				fmt.Fprintln(w, `{"data": {"me":{ "name": "a8m", "org": "graph"}}}`)
 			case strings.Contains(m.Query, "query"):
 				// Checksum before archiving.
@@ -879,6 +972,7 @@ env {
 		require.NoError(t, err, "target id is not a valid uuid")
 		require.False(t, report.StartTime.IsZero())
 		require.False(t, report.EndTime.IsZero())
+		require.Nil(t, report.Context)
 		require.Equal(t, cloudapi.ReportMigrationInput{
 			ProjectName:  "example",
 			DirName:      "migrations/v1/mysql",
@@ -896,6 +990,24 @@ env {
 		}, report)
 	})
 
+	t.Run("WithContext", func(t *testing.T) {
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--url", u,
+			"--var", "cloud_url="+srv.URL,
+			"--context", `{ "triggerType": "KUBERNETES", "triggerVersion": "v1.2.3" }`,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "No migration files to execute\nhttps://gh.atlasgo.cloud/deployments/51539607559\n", s)
+		require.NotNil(t, report.Context)
+		require.Equal(t, report.Context.TriggerType, "KUBERNETES")
+		require.Equal(t, report.Context.TriggerVersion, "v1.2.3")
+	})
+
 	t.Run("WithFiles", func(t *testing.T) {
 		require.NoError(t, dir.WriteFile("1.sql", []byte("create table foo (id int)")))
 		require.NoError(t, dir.WriteFile("2.sql", []byte("create table bar (id int)")))
@@ -908,11 +1020,13 @@ env {
 			"--env", "local",
 			"--url", u,
 			"--var", "cloud_url="+srv.URL,
+			// Inject fake variable to enforce re-evaluation of the data source (skip cache).
+			"--var", fmt.Sprintf("cache=%s", uuid.NewString()),
 		)
 		require.NoError(t, err)
 		// Reporting does not affect the output.
 		require.True(t, strings.HasPrefix(s, "Migrating to version 2 (2 migrations in total):"))
-		require.True(t, strings.HasSuffix(s, "  -- 2 migrations \n  -- 2 sql statements\nhttps://gh.atlasgo.cloud/deployments/51539607559\n"))
+		require.True(t, strings.HasSuffix(s, "  -- 2 migrations\n  -- 2 sql statements\nhttps://gh.atlasgo.cloud/deployments/51539607559\n"))
 		require.Equal(t, "", report.FromVersion, "from empty database")
 		require.Equal(t, "2", report.ToVersion)
 		require.Equal(t, "2", report.CurrentVersion)
@@ -936,10 +1050,12 @@ env {
 			"--env", "local",
 			"--url", u,
 			"--var", "cloud_url="+srv.URL,
+			// Inject fake variable to enforce re-evaluation of the data source (skip cache).
+			"--var", fmt.Sprintf("cache=%s", uuid.NewString()),
 		)
 		require.NoError(t, err)
 		// Reporting error should not affect the migration execution.
-		require.True(t, strings.HasSuffix(s, "  -- 1 migrations \n  -- 1 sql statements\nError: unexpected status code: 500\n"))
+		require.True(t, strings.HasSuffix(s, "  -- 1 migration\n  -- 1 sql statement\nError: unexpected status code: 500\n"))
 
 		// Custom logging.
 		cmd = migrateCmd()
@@ -968,9 +1084,13 @@ env {
 			"--env", "local",
 			"--url", "openerror://db",
 			"--var", "cloud_url="+srv.URL,
+			"--context", `{ "triggerType": "KUBERNETES", "triggerVersion": "v1.2.3" }`,
 		)
 		require.EqualError(t, err, "openerror")
 		require.Equal(t, "Error: openerror", *reports.Error)
+		require.NotNil(t, reports.Context)
+		require.Equal(t, reports.Context.TriggerType, "KUBERNETES")
+		require.Equal(t, reports.Context.TriggerVersion, "v1.2.3")
 	})
 
 	t.Run("RedactedURL", func(t *testing.T) {
@@ -1019,7 +1139,7 @@ func TestMigrate_ApplyCloudReportSet(t *testing.T) {
 			}
 			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
 			switch {
-			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "bot"):
+			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "me"):
 				fmt.Fprintln(w, `{"data": {"me":{ "name": "a8m", "org": "graph"}}}`)
 			case strings.Contains(m.Query, "query"):
 				// Checksum before archiving.
@@ -1116,6 +1236,25 @@ env {
 		require.Contains(t, report.Log[2].Log[1].Text, "Migration directory: mem://migration_set")
 	})
 
+	t.Run("WithContext", func(t *testing.T) {
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--context", `{ "triggerType": "KUBERNETES", "triggerVersion": "v1.2.3" }`,
+			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
+			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
+			"--var", "cloud_url="+srv.URL,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "No migration files to execute\nNo migration files to execute\nhttps://gh.atlasgo.cloud/deployments/sets/94489280524\n", s)
+		require.NotNil(t, report.Context)
+		require.Equal(t, report.Context.TriggerType, "KUBERNETES")
+		require.Equal(t, report.Context.TriggerVersion, "v1.2.3")
+	})
+
 	t.Run("Error", func(t *testing.T) {
 		require.NoError(t, dir.WriteFile("1.sql", []byte("create table foo (id int)")))
 		require.NoError(t, dir.WriteFile("2.sql", []byte("create table bar (id int)")))
@@ -1131,7 +1270,7 @@ env {
 			"--var", "cloud_url="+srv.URL,
 			"--allow-dirty",
 		)
-		require.EqualError(t, err, `sql/migrate: execute: executing statement "create table bar (id int)" from version "2": table bar already exists`)
+		require.EqualError(t, err, `sql/migrate: executing statement "create table bar (id int)" from version "2": table bar already exists`)
 		require.Contains(t, s, "Migrating to version 2 (2 migrations in total):")
 		require.Contains(t, s, "Error: sql/migrate:")
 		require.NotEmpty(t, report.ID)
@@ -1142,7 +1281,7 @@ env {
 		require.Equal(t, 2, report.Planned)
 		require.Len(t, report.Completed, 1)
 		require.NotNil(t, report.Error)
-		require.Equal(t, `Error: sql/migrate: execute: executing statement "create table bar (id int)" from version "2": table bar already exists`, *report.Error)
+		require.Equal(t, `Error: sql/migrate: executing statement "create table bar (id int)" from version "2": table bar already exists`, *report.Error)
 
 		// Verify summary log.
 		require.Len(t, report.Log, 2)
@@ -1154,7 +1293,7 @@ env {
 		require.True(t, report.Log[1].Error)
 		require.Contains(t, report.Log[1].Log[0].Text, "Target URL: sqlite://file")
 		require.Contains(t, report.Log[1].Log[1].Text, "Migration directory: mem://migration_set")
-		require.Equal(t, `Error: sql/migrate: execute: executing statement "create table bar (id int)" from version "2": table bar already exists`, report.Log[1].Log[2].Text)
+		require.Equal(t, `Error: sql/migrate: executing statement "create table bar (id int)" from version "2": table bar already exists`, report.Log[1].Log[2].Text)
 	})
 }
 
@@ -1223,8 +1362,7 @@ func TestMigrate_Diff(t *testing.T) {
 
 	t.Run("Edit", func(t *testing.T) {
 		p := t.TempDir()
-		require.NoError(t, os.Setenv("EDITOR", "echo '-- Comment' >>"))
-		t.Cleanup(func() { require.NoError(t, os.Unsetenv("EDITOR")) })
+		t.Setenv("EDITOR", "echo '-- Comment' >>")
 		args := []string{
 			"--edit",
 			"--dir", "file://" + p,
@@ -1925,7 +2063,6 @@ table "table" {
       table.table.column.col,
       table.table.column.age,
     ]
-    comment = "index comment"
   }
   foreign_key "accounts" {
     columns = [
