@@ -77,7 +77,7 @@ migration.`,
   atlas schema apply -u "mysql://localhost" --to "file://schema.sql" --dev-url "docker://mysql/8/dev"
   atlas schema apply --env local --dev-url "docker://postgres/15/dev?search_path=public" --dry-run
   atlas schema apply -u "sqlite://file.db" --to "file://schema.sql" --dev-url "sqlite://dev?mode=memory"`,
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				switch {
 				case GlobalFlags.SelectedEnv == "":
 					env, err := selectEnv(cmd)
@@ -86,7 +86,7 @@ migration.`,
 					}
 					return schemaApplyRun(cmd, flags, env)
 				default:
-					_, envs, err := EnvByName(cmd, GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
+					_, envs, err := EnvByName(cmd, GlobalFlags.SelectedEnv, GlobalFlags.Vars)
 					if err != nil {
 						return err
 					}
@@ -94,7 +94,7 @@ migration.`,
 						return schemaApplyRun(cmd, flags, e)
 					})
 				}
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -126,6 +126,8 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 		return errors.New(`--log and --format can only be used with --dry-run or --auto-approve`)
 	case flags.txMode != txModeNone && flags.txMode != txModeFile:
 		return fmt.Errorf("unknown tx-mode %q", flags.txMode)
+	case flags.autoApprove && env.Lint.Review != "":
+		return fmt.Errorf("auto-approve is not allowed when a lint policy is set to %q", env.Lint.Review)
 	}
 	// If the old -f flag is given convert them to the URL format. If both are given,
 	// cobra would throw an error since they are marked as mutually exclusive.
@@ -152,7 +154,7 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 		}
 		defer dev.Close()
 	}
-	from, err := stateReader(ctx, &stateReaderConfig{
+	from, err := stateReader(ctx, env, &stateReaderConfig{
 		urls:    []string{flags.url},
 		schemas: flags.schemas,
 		exclude: flags.exclude,
@@ -165,7 +167,7 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 	if !ok {
 		return errors.New("--url must be a database connection")
 	}
-	to, err := stateReader(ctx, &stateReaderConfig{
+	to, err := stateReader(ctx, env, &stateReaderConfig{
 		urls:    flags.toURLs,
 		dev:     dev,
 		client:  client,
@@ -274,9 +276,9 @@ As a safety feature, 'atlas schema clean' will ask for confirmation before attem
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				return schemaFlagsFromConfig(cmd)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				return schemaCleanRun(cmd, args, flags)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -361,13 +363,13 @@ The database states can be read from a connected database, an HCL project or a m
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				return schemaFlagsFromConfig(cmd)
 			},
-			RunE: func(cmd *cobra.Command, args []string) error {
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
 				env, err := selectEnv(cmd)
 				if err != nil {
 					return err
 				}
 				return schemaDiffRun(cmd, args, flags, env)
-			},
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -396,7 +398,7 @@ func schemaDiffRun(cmd *cobra.Command, _ []string, flags schemaDiffFlags, env *E
 		}
 		defer c.Close()
 	}
-	from, err := stateReader(ctx, &stateReaderConfig{
+	from, err := stateReader(ctx, env, &stateReaderConfig{
 		urls:    flags.fromURL,
 		dev:     c,
 		vars:    GlobalFlags.Vars,
@@ -407,7 +409,7 @@ func schemaDiffRun(cmd *cobra.Command, _ []string, flags schemaDiffFlags, env *E
 		return err
 	}
 	defer from.Close()
-	to, err := stateReader(ctx, &stateReaderConfig{
+	to, err := stateReader(ctx, env, &stateReaderConfig{
 		urls:    flags.toURL,
 		dev:     c,
 		vars:    GlobalFlags.Vars,
@@ -458,6 +460,7 @@ func schemaInspectCmd() *cobra.Command {
 
 func schemaInspectCmdWithFlags() (*cobra.Command, *schemaInspectFlags) {
 	var (
+		env   *Env
 		flags schemaInspectFlags
 		cmd   = &cobra.Command{
 			Use:   "inspect",
@@ -479,12 +482,15 @@ flag.
   atlas schema inspect -u "mariadb://user:pass@localhost:3306/" --schema=schemaA,schemaB -s schemaC
   atlas schema inspect --url "postgres://user:pass@host:port/dbname?sslmode=disable"
   atlas schema inspect -u "sqlite://file:ex1.db?_fk=1"`,
-			PreRunE: func(cmd *cobra.Command, _ []string) error {
-				return schemaFlagsFromConfig(cmd)
-			},
-			RunE: func(cmd *cobra.Command, args []string) error {
-				return schemaInspectRun(cmd, args, flags)
+			PreRunE: func(cmd *cobra.Command, args []string) (err error) {
+				if env, err = selectEnv(cmd); err != nil {
+					return err
+				}
+				return setSchemaEnvFlags(cmd, env)
 			},
+			RunE: RunE(func(cmd *cobra.Command, args []string) error {
+				return schemaInspectRun(cmd, args, flags, env)
+			}),
 		}
 	)
 	cmd.Flags().SortFlags = false
@@ -499,12 +505,12 @@ flag.
 	return cmd, &flags
 }
 
-func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags) error {
+func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags, env *Env) error {
 	var (
 		ctx = cmd.Context()
 		dev *sqlclient.Client
 	)
-	useDev, err := readerUseDev(flags.url)
+	useDev, err := readerUseDev(env, flags.url)
 	if err != nil {
 		return err
 	}
@@ -514,7 +520,7 @@ func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags)
 		}
 		defer dev.Close()
 	}
-	r, err := stateReader(ctx, &stateReaderConfig{
+	r, err := stateReader(ctx, env, &stateReaderConfig{
 		urls:    []string{flags.url},
 		dev:     dev,
 		vars:    GlobalFlags.Vars,
@@ -558,9 +564,9 @@ Unless stated otherwise, the fmt command will use the current directory.
 After running, the command will print the names of the files it has formatted. If all
 files in the directory are formatted, no input will be printed out.
 `,
-		RunE: func(cmd *cobra.Command, args []string) error {
+		RunE: RunE(func(cmd *cobra.Command, args []string) error {
 			return schemaFmtRun(cmd, args)
-		},
+		}),
 	}
 	return cmd
 }
@@ -594,7 +600,7 @@ func selectEnv(cmd *cobra.Command) (*Env, error) {
 	switch name := GlobalFlags.SelectedEnv; {
 	// A config file was passed without an env.
 	case name == "" && cmd.Flags().Changed(flagConfig):
-		p, envs, err := EnvByName(cmd, name, WithInput(GlobalFlags.Vars))
+		p, envs, err := EnvByName(cmd, name, GlobalFlags.Vars)
 		if err != nil {
 			return nil, err
 		}
@@ -607,7 +613,7 @@ func selectEnv(cmd *cobra.Command) (*Env, error) {
 		return &Env{Lint: &Lint{}, Migration: &Migration{}}, nil
 	// Env was passed.
 	default:
-		_, envs, err := EnvByName(cmd, name, WithInput(GlobalFlags.Vars))
+		_, envs, err := EnvByName(cmd, name, GlobalFlags.Vars)
 		if err != nil {
 			return nil, err
 		}
@@ -640,6 +646,11 @@ func setSchemaEnvFlags(cmd *cobra.Command, env *Env) error {
 	if err != nil {
 		return err
 	}
+	for i, s := range srcs {
+		if !isURL(s) {
+			srcs[i] = "file://" + s
+		}
+	}
 	if err := maySetFlag(cmd, flagFile, strings.Join(srcs, ",")); err != nil {
 		return err
 	}
@@ -658,6 +669,10 @@ func setSchemaEnvFlags(cmd *cobra.Command, env *Env) error {
 		if err := maySetFlag(cmd, flagFormat, env.Format.Schema.Diff); err != nil {
 			return err
 		}
+	case "push":
+		if err := maySetFlag(cmd, flagTo, strings.Join(srcs, ",")); err != nil {
+			return err
+		}
 	}
 	return nil
 }
@@ -725,9 +740,10 @@ const (
 // cmdPrompt returns a promptui.Select that uses the given command's input and output.
 func cmdPrompt(cmd *cobra.Command) *promptui.Select {
 	return &promptui.Select{
-		Label:  "Are you sure?",
-		Stdin:  io.NopCloser(cmd.InOrStdin()),
-		Stdout: nopCloser{cmd.OutOrStdout()},
+		Label:    "Are you sure?",
+		HideHelp: true,
+		Stdin:    io.NopCloser(cmd.InOrStdin()),
+		Stdout:   nopCloser{cmd.OutOrStdout()},
 	}
 }
 
