@@ -19,7 +19,6 @@ import (
 	"path"
 	"path/filepath"
 	"reflect"
-	"strconv"
 	"text/template"
 	"time"
 
@@ -34,37 +33,35 @@ import (
 	"golang.org/x/oauth2/google"
 	sqladmin "google.golang.org/api/sqladmin/v1beta4"
 
-	"github.com/zclconf/go-cty/cty/gocty"
-
-	"entgo.io/ent/dialect/sql"
-	entschema "entgo.io/ent/dialect/sql/schema"
-	"entgo.io/ent/entc"
-	"entgo.io/ent/entc/gen"
 	"github.com/hashicorp/hcl/v2"
 	"github.com/hashicorp/hcl/v2/gohcl"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
 	"github.com/zclconf/go-cty/cty"
-	"gocloud.dev/gcerrors"
+	"github.com/zclconf/go-cty/cty/gocty"
 	"gocloud.dev/runtimevar"
 	_ "gocloud.dev/runtimevar/awssecretsmanager"
 	_ "gocloud.dev/runtimevar/constantvar"
-	"gocloud.dev/runtimevar/driver"
 	_ "gocloud.dev/runtimevar/filevar"
 	_ "gocloud.dev/runtimevar/gcpsecretmanager"
 	_ "gocloud.dev/runtimevar/httpvar"
 )
 
-// DataSources exposes the data sources provided by this package.
-var DataSources = []schemahcl.Option{
-	schemahcl.WithDataSource("sql", Query),
-	schemahcl.WithDataSource("runtimevar", RuntimeVar),
-	schemahcl.WithDataSource("template_dir", TemplateDir),
-	schemahcl.WithDataSource("remote_dir", RemoteDir),
-	schemahcl.WithDataSource("hcl_schema", SchemaHCL),
-	schemahcl.WithDataSource("external_schema", SchemaExternal),
-	schemahcl.WithDataSource("aws_rds_token", AWSRDSToken),
-	schemahcl.WithDataSource("gcp_cloudsql_token", GCPCloudSQLToken),
-}
+// SpecOptions exposes the schema spec options like data-sources provided by this package.
+var SpecOptions = append(
+	[]schemahcl.Option{
+		schemahcl.WithDataSource("sql", Query),
+		schemahcl.WithDataSource("external", External),
+		schemahcl.WithDataSource("runtimevar", RuntimeVar),
+		schemahcl.WithDataSource("template_dir", TemplateDir),
+		schemahcl.WithDataSource("remote_dir", RemoteDir),
+		schemahcl.WithDataSource("remote_schema", RemoteSchema),
+		schemahcl.WithDataSource("hcl_schema", SchemaHCL),
+		schemahcl.WithDataSource("external_schema", SchemaExternal),
+		schemahcl.WithDataSource("aws_rds_token", AWSRDSToken),
+		schemahcl.WithDataSource("gcp_cloudsql_token", GCPCloudSQLToken),
+	},
+	specOptions...,
+)
 
 // RuntimeVar exposes the gocloud.dev/runtimevar as a schemahcl datasource.
 //
@@ -271,6 +268,59 @@ func Query(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error) {
 	return cty.ObjectVal(obj), nil
 }
 
+// External allows loading data using external program execution.
+//
+//	data "external" "env1" {
+//	  program = [
+//	    "node",
+//	    loadenv.js",
+//	  ]
+//	}
+//
+//	data "external" "env2" {
+//	  program = [
+//	    "bash",
+//	    "-c",
+//	    "env_to_json --file=${var.envfile} | jq '...' ",
+//	  ]
+//	}
+func External(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error) {
+	var (
+		args struct {
+			Program []string `hcl:"program"`
+			Dir     string   `hcl:"working_dir,optional"`
+			Remain  hcl.Body `hcl:",remain"`
+		}
+		errorf = blockError("data.external", block)
+	)
+	if diags := gohcl.DecodeBody(block.Body, ctx, &args); diags.HasErrors() {
+		return cty.NilVal, errorf("decoding body: %v", diags)
+	}
+	attrs, diags := args.Remain.JustAttributes()
+	if diags.HasErrors() {
+		return cty.NilVal, errorf("getting attributes: %v", diags)
+	}
+	if len(attrs) > 0 {
+		return cty.NilVal, errorf("unexpected attributes: %v", attrs)
+	}
+	if len(args.Program) == 0 {
+		return cty.NilVal, errorf("program cannot be empty")
+	}
+	cmd := exec.Command(args.Program[0], args.Program[1:]...)
+	if args.Dir != "" {
+		cmd.Dir = args.Dir
+	}
+	out, err := cmd.Output()
+	if err != nil {
+		msg := err.Error()
+		if err1 := (*exec.ExitError)(nil); errors.As(err, &err1) && len(err1.Stderr) > 0 {
+			msg = string(err1.Stderr)
+		}
+		return cty.NilVal, errorf("running program %v: %v", cmd.Path, msg)
+	}
+	return cty.StringVal(string(out)), nil
+}
+
 // TemplateDir implements migrate.Dir interface for template directories.
 //
 //	data "template_dir" "name" {
@@ -413,7 +463,7 @@ func TemplateDir(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error
 	u := fmt.Sprintf("mem://%s", dirname)
 	// Allow using reading the computed dir as a state source.
 	memLoader.states[u] = StateLoaderFunc(func(ctx context.Context, config *StateReaderConfig) (*StateReadCloser, error) {
-		return stateReaderSQL(ctx, config, dir)
+		return stateReaderSQL(ctx, config, dir, nil, nil)
 	})
 	return cty.ObjectVal(map[string]cty.Value{
 		"url": cty.StringVal(u),
@@ -580,12 +630,15 @@ func SchemaExternal(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, er
 	out, err := cmd.Output()
 	if err != nil {
 		msg := err.Error()
-		if ee, ok := err.(*exec.ExitError); ok && len(ee.Stderr) > 0 {
-			msg = string(ee.Stderr)
+		if err1 := (*exec.ExitError)(nil); errors.As(err, &err1) && len(err1.Stderr) > 0 {
+			msg = string(err1.Stderr)
 		}
 		return cty.NilVal, errorf("running program %v: %v", cmd.Path, msg)
 	}
-	dir, err := fileAsDir("schema.sql", out)
+	// Directory files must have an .sql extension to be read by the executor.
+	// The "schema" word is added to indicate that unlike data-source errors, load error
+	// comes from the output of the data-source (SQL representation of the state/schema).
+	dir, err := filesAsDir(migrate.NewLocalFile(fmt.Sprintf("%s/schema.sql", block.Labels[1]), out))
 	if err != nil {
 		return cty.NilVal, errorf("converting output to migration: %v", err)
 	}
@@ -594,7 +647,7 @@ func SchemaExternal(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, er
 		return cty.NilVal, errorf("build url: %v", err)
 	}
 	memLoader.states[u] = StateLoaderFunc(func(ctx context.Context, config *StateReaderConfig) (*StateReadCloser, error) {
-		return stateReaderSQL(ctx, config, dir)
+		return stateSchemaSQL(ctx, config, dir)
 	})
 	return cty.ObjectVal(map[string]cty.Value{
 		"url": cty.StringVal(u),
@@ -720,125 +773,6 @@ func (l MemLoader) LoadState(ctx context.Context, config *StateReaderConfig) (*S
 	return l.states[u].LoadState(ctx, config)
 }
 
-// EntLoader is a StateLoader for loading ent.Schema's as StateReader's.
-type EntLoader struct{}
-
-// LoadState returns a migrate.StateReader that reads the schema from an ent.Schema.
-func (l EntLoader) LoadState(ctx context.Context, config *StateReaderConfig) (*StateReadCloser, error) {
-	switch {
-	case len(config.URLs) != 1:
-		return nil, errors.New(`"ent://" requires exactly one schema URL`)
-	case config.Dev == nil:
-		return nil, errors.New(`required flag "--dev-url" not set`)
-	case config.Dev.URL.Schema == "":
-		return nil, errNotSchemaURL
-	case config.URLs[0].Query().Has("globalid"):
-		return nil, errors.New("globalid is not supported by this command. Use 'migrate diff' instead")
-	}
-	tables, err := l.tables(config.URLs[0])
-	if err != nil {
-		return nil, err
-	}
-	m, err := entschema.NewMigrate(sql.OpenDB(config.Dev.Name, config.Dev.DB))
-	if err != nil {
-		return nil, fmt.Errorf("creating migrate reader: %w", err)
-	}
-	realm, err := m.StateReader(tables...).ReadState(ctx)
-	if err != nil {
-		return nil, fmt.Errorf("reading schema state: %w", err)
-	}
-	if len(realm.Schemas) != 1 {
-		return nil, fmt.Errorf("expect exactly one schema, got %d", len(realm.Schemas))
-	}
-	if nr, ok := config.Dev.Driver.(schema.Normalizer); ok {
-		if realm.Schemas[0], err = nr.NormalizeSchema(ctx, realm.Schemas[0]); err != nil {
-			return nil, err
-		}
-	}
-	// Use the dev-database schema name if the schema name is empty.
-	if realm.Schemas[0].Name == "" && config.Dev.URL.Schema != "" {
-		realm.Schemas[0].Name = config.Dev.URL.Schema
-	}
-	for _, t := range realm.Schemas[0].Tables {
-		t.Schema = realm.Schemas[0]
-	}
-	rc := &StateReadCloser{StateReader: migrate.Realm(realm)}
-	if config.Dev != nil && config.Dev.URL.Schema != "" {
-		rc.Schema = config.Dev.URL.Schema
-	}
-	return rc, nil
-}
-
-// MigrateDiff returns the diff between ent.Schema and a directory.
-func (l EntLoader) MigrateDiff(ctx context.Context, opts *MigrateDiffOptions) error {
-	if !l.needDiff(opts.To) {
-		return errors.New("invalid diff call")
-	}
-	if opts.Dev.URL.Schema == "" {
-		return errNotSchemaURL
-	}
-	u, err := sqlclient.ParseURL(opts.To[0])
-	if err != nil {
-		return nil
-	}
-	tables, err := l.tables(u)
-	if err != nil {
-		return err
-	}
-	m, err := entschema.NewMigrate(
-		sql.OpenDB(opts.Dev.Name, opts.Dev.DB),
-		entschema.WithDir(opts.Dir),
-		entschema.WithDropColumn(true),
-		entschema.WithDropIndex(true),
-		entschema.WithErrNoPlan(true),
-		entschema.WithFormatter(dirFormatter(opts.Dir)),
-		entschema.WithGlobalUniqueID(true),
-		entschema.WithIndent(opts.Indent),
-		entschema.WithMigrationMode(entschema.ModeReplay),
-		entschema.WithDiffOptions(opts.Options...),
-	)
-	if err != nil {
-		return fmt.Errorf("creating migrate reader: %w", err)
-	}
-	return m.NamedDiff(ctx, opts.Name, tables...)
-}
-
-// needDiff indicates if we need to offload the diffing to Ent in
-// case global unique id is enabled in versioned migration mode.
-func (EntLoader) needDiff(to []string) bool {
-	if len(to) != 1 {
-		return false
-	}
-	u1, err := url.Parse(to[0])
-	if err != nil || u1.Scheme != "ent" {
-		return false
-	}
-	gid, _ := strconv.ParseBool(u1.Query().Get("globalid"))
-	return gid
-}
-
-func (EntLoader) tables(u *url.URL) ([]*entschema.Table, error) {
-	abs, err := filepath.Abs(filepath.Join(u.Host, u.Path))
-	if err != nil {
-		return nil, err
-	}
-	var opts []entc.Option
-	if tags, ok := u.Query()["build-tags"]; ok {
-		opts = append(opts, entc.BuildTags(tags...))
-	}
-	cfg := &gen.Config{}
-	for _, opt := range opts {
-		if err = opt(cfg); err != nil {
-			return nil, err
-		}
-	}
-	graph, err := entc.LoadGraph(abs, cfg)
-	if err != nil {
-		return nil, fmt.Errorf("loading schema: %w", err)
-	}
-	return graph.Tables()
-}
-
 func dirFormatter(dir migrate.Dir) migrate.Formatter {
 	switch dir.(type) {
 	case *sqltool.DBMateDir:
@@ -855,68 +789,3 @@ func dirFormatter(dir migrate.Dir) migrate.Formatter {
 		return migrate.DefaultFormatter
 	}
 }
-
-func init() {
-	runtimevar.DefaultURLMux().RegisterVariable(Scheme, &URLOpener{})
-}
-
-// Scheme is the URL scheme envvar registers its URLOpener under on runtimevar.DefaultURLMux.
-const Scheme = "env"
-
-// URLOpener opens envvar URLs like "env://VARIABLE".
-type URLOpener struct {
-}
-
-// OpenVariableURL opens the variable at the URL's path. See the package doc
-// for more details.
-func (o *URLOpener) OpenVariableURL(_ context.Context, u *url.URL) (*runtimevar.Variable, error) {
-	return New(os.Getenv(u.Host)), nil
-}
-
-// New constructs a *runtimevar.Variable holding value.
-func New(value string) *runtimevar.Variable {
-	return runtimevar.New(&watcher{value: value, t: time.Now()})
-}
-
-// watcher implements driver.Watcher and driver.State.
-type watcher struct {
-	value string
-	err   error
-	t     time.Time
-}
-
-// Value implements driver.State.Value.
-func (w *watcher) Value() (interface{}, error) {
-	return w.value, w.err
-}
-
-// UpdateTime implements driver.State.UpdateTime.
-func (w *watcher) UpdateTime() time.Time {
-	return w.t
-}
-
-// As implements driver.State.As.
-func (w *watcher) As(any) bool { return false }
-
-// WatchVariable implements driver.WatchVariable.
-func (w *watcher) WatchVariable(ctx context.Context, prev driver.State) (driver.State, time.Duration) {
-	// The first time this is called, return the constant value.
-	if prev == nil {
-		return w, 0
-	}
-	// On subsequent calls, block forever as the value will never change.
-	<-ctx.Done()
-	w.err = ctx.Err()
-	return w, 0
-}
-
-// Close implements driver.Close.
-func (*watcher) Close() error { return nil }
-
-// ErrorAs implements driver.ErrorAs.
-func (*watcher) ErrorAs(error, any) bool { return false }
-
-// ErrorCode implements driver.ErrorCode
-func (*watcher) ErrorCode(error) gcerrors.ErrorCode {
-	return gcerrors.Unknown
-}
