@@ -9,6 +9,7 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"slices"
 	"sort"
 	"strings"
 	"text/template"
@@ -19,7 +20,6 @@ import (
 	"ariga.io/atlas/sql/sqlclient"
 
 	"github.com/fatih/color"
-	"github.com/olekukonko/tablewriter"
 )
 
 var (
@@ -52,6 +52,21 @@ type (
 	// File wraps migrate.File to implement json.Marshaler.
 	File struct{ migrate.File }
 
+	// FileChecks represents a set of checks to run before applying a file.
+	FileChecks struct {
+		Name  string     `json:"Name,omitempty"`  // File/group name.
+		Stmts []*Check   `json:"Stmts,omitempty"` // Checks statements executed.
+		Error *StmtError `json:"Error,omitempty"` // Assertion error.
+		Start time.Time  `json:"Start,omitempty"` // Start assertion time.
+		End   time.Time  `json:"End,omitempty"`   // End assertion time.
+	}
+
+	// Check represents an assertion and its status.
+	Check struct {
+		Stmt  string  `json:"Stmt,omitempty"`  // Assertion statement.
+		Error *string `json:"Error,omitempty"` // Assertion error, if any.
+	}
+
 	// StmtError groups a statement with its execution error.
 	StmtError struct {
 		Stmt string `json:"Stmt,omitempty"` // SQL statement that failed.
@@ -95,23 +110,32 @@ var (
 	StatusTemplateFuncs = merge(template.FuncMap{
 		"json":       jsonEncode,
 		"json_merge": jsonMerge,
-		"table":      table,
 		"default": func(report *MigrateStatus) (string, error) {
 			var buf bytes.Buffer
-			t, err := template.New("report").Funcs(ColorTemplateFuncs).Parse(`Migration Status:
-{{- if eq .Status "OK"      }} {{ green .Status }}{{ end }}
+			t, err := template.New("report").
+				Funcs(template.FuncMap{
+					"add": add,
+				}).
+				Funcs(ColorTemplateFuncs).
+				Parse(`Migration Status:
+{{- if eq .Status "OK" }} {{ green .Status }}{{ end }}
 {{- if eq .Status "PENDING" }} {{ yellow .Status }}{{ end }}
   {{ yellow "--" }} Current Version: {{ cyan .Current }}
 {{- if gt .Total 0 }}{{ printf " (%s statements applied)" (yellow "%d" .Count) }}{{ end }}
-  {{ yellow "--" }} Next Version:    {{ cyan .Next }}{{ if .FromCheckpoint }} (checkpoint){{ end }}
+  {{ yellow "--" }} Next Version:    {{ if .Next }}{{ cyan .Next }}{{ if .FromCheckpoint }} (checkpoint){{ end }}{{ else }}UNKNOWN{{ end }}
 {{- if gt .Total 0 }}{{ printf " (%s statements left)" (yellow "%d" .Left) }}{{ end }}
   {{ yellow "--" }} Executed Files:  {{ len .Applied }}{{ if gt .Total 0 }} (last one partially){{ end }}
-  {{ yellow "--" }} Pending Files:   {{ len .Pending }}
-{{ if gt .Total 0 }}
+  {{ yellow "--" }} Pending Files:   {{ add (len .Pending) (len .OutOfOrder) }}{{ if .OutOfOrder }} ({{ if .Pending }}{{ len .OutOfOrder }} {{ end }}out of order){{ end }}
+{{- if gt .Total 0 }}
+
 Last migration attempt had errors:
   {{ yellow "--" }} SQL:   {{ .SQL }}
   {{ yellow "--" }} {{ red "ERROR:" }} {{ .Error }}
-{{ end }}`)
+{{- else if and .OutOfOrder .Error }}
+
+  {{ red "ERROR:" }} {{ .Error }}
+{{- end }}
+`)
 			if err != nil {
 				return "", err
 			}
@@ -128,16 +152,17 @@ Last migration attempt had errors:
 type MigrateStatus struct {
 	context.Context `json:"-"`
 	Env             `json:"Env"`
-	Available       Files               `json:"Available,omitempty"` // Available migration files
-	Pending         Files               `json:"Pending,omitempty"`   // Pending migration files
-	Applied         []*migrate.Revision `json:"Applied,omitempty"`   // Applied migration files
-	Current         string              `json:"Current,omitempty"`   // Current migration version
-	Next            string              `json:"Next,omitempty"`      // Next migration version
-	Count           int                 `json:"Count,omitempty"`     // Count of applied statements of the last revision
-	Total           int                 `json:"Total,omitempty"`     // Total statements of the last migration
-	Status          string              `json:"Status,omitempty"`    // Status of migration (OK, PENDING)
-	Error           string              `json:"Error,omitempty"`     // Last Error that occurred
-	SQL             string              `json:"SQL,omitempty"`       // SQL that caused the last Error
+	Available       Files               `json:"Available,omitempty"`  // Available migration files
+	OutOfOrder      Files               `json:"OutOfOrder,omitempty"` // OutOfOrder migration files
+	Pending         Files               `json:"Pending,omitempty"`    // Pending migration files
+	Applied         []*migrate.Revision `json:"Applied,omitempty"`    // Applied migration files
+	Current         string              `json:"Current,omitempty"`    // Current migration version
+	Next            string              `json:"Next,omitempty"`       // Next migration version
+	Count           int                 `json:"Count,omitempty"`      // Count of applied statements of the last revision
+	Total           int                 `json:"Total,omitempty"`      // Total statements of the last migration
+	Status          string              `json:"Status,omitempty"`     // Status of migration (OK, PENDING)
+	Error           string              `json:"Error,omitempty"`      // Last Error that occurred
+	SQL             string              `json:"SQL,omitempty"`        // SQL that caused the last Error
 }
 
 // Left returns the amount of statements left to apply (if any).
@@ -153,56 +178,6 @@ func (r *MigrateStatus) FromCheckpoint() bool {
 	return ok && ck.IsCheckpoint()
 }
 
-func table(report *MigrateStatus) (string, error) {
-	var buf strings.Builder
-	tbl := tablewriter.NewWriter(&buf)
-	tbl.SetRowLine(true)
-	tbl.SetAutoMergeCellsByColumnIndex([]int{0})
-	tbl.SetHeader([]string{
-		"Version",
-		"Description",
-		"Status",
-		"Count",
-		"Executed At",
-		"Execution Time",
-		"Error",
-		"SQL",
-	})
-	for _, r := range report.Applied {
-		tbl.Append([]string{
-			r.Version,
-			r.Description,
-			r.Type.String(),
-			fmt.Sprintf("%d/%d", r.Applied, r.Total),
-			r.ExecutedAt.Format("2006-01-02 15:04:05 MST"),
-			r.ExecutionTime.String(),
-			r.Error,
-			r.ErrorStmt,
-		})
-	}
-	for i, f := range report.Pending {
-		var c string
-		if i == 0 {
-			if r := report.Applied[len(report.Applied)-1]; f.Version() == r.Version && r.Applied < r.Total {
-				stmts, err := f.Stmts()
-				if err != nil {
-					return "", err
-				}
-				c = fmt.Sprintf("%d/%d", len(stmts)-r.Applied, len(stmts))
-			}
-		}
-		tbl.Append([]string{
-			f.Version(),
-			f.Desc(),
-			"pending",
-			c,
-			"", "", "", "",
-		})
-	}
-	tbl.Render()
-	return buf.String(), nil
-}
-
 // MigrateSetTemplate holds the default template of the 'migrate set' command.
 var MigrateSetTemplate = template.Must(template.New("set").
 	Funcs(ColorTemplateFuncs).Parse(`
@@ -286,10 +261,11 @@ func (r *RevisionOp) ColoredVersion() string {
 var (
 	// ApplyTemplateFuncs are global functions available in apply report templates.
 	ApplyTemplateFuncs = merge(ColorTemplateFuncs, template.FuncMap{
-		"dec":        dec,
+		"add":        add,
 		"upper":      strings.ToUpper,
 		"json":       jsonEncode,
 		"json_merge": jsonMerge,
+		"indent_ln":  indentLn,
 	})
 
 	// MigrateApplyTemplate holds the default template of the 'migrate apply' command.
@@ -297,30 +273,45 @@ var (
 				New("report").
 				Funcs(ApplyTemplateFuncs).
 				Parse(`{{- if not .Pending -}}
-No migration files to execute
+{{- println "No migration files to execute" }}
 {{- else -}}
 Migrating to version {{ cyan .Target }}{{ with .Current }} from {{ cyan . }}{{ end }} ({{ len .Pending }} migrations in total):
 {{ range $i, $f := .Applied }}
-  {{ yellow "--" }} migrating version {{ cyan $f.File.Version }}{{ range $f.Applied }}
-    {{ cyan "->" }} {{ . }}{{ end }}
-  {{- with .Error }}
-    {{ redBgWhiteFg .Text }}
-  {{- else }}
-  {{ yellow "--" }} ok ({{ yellow (.End.Sub .Start).String }})
-  {{- end }}
-{{ end }}
-  {{ cyan "-------------------------" }}
-  {{ yellow "--" }} {{ .End.Sub .Start }}
-{{- $files := len .Applied }}
-{{- $stmts := .CountStmts }}
-{{- if .Error }}
-  {{ yellow "--" }} {{ dec $files }} migrations ok (1 with errors)
-  {{ yellow "--" }} {{ dec $stmts }} sql statements ok (1 with errors)
-{{- else }}
-  {{ yellow "--" }} {{ len .Applied }} migrations 
-  {{ yellow "--" }} {{ .CountStmts  }} sql statements
-{{- end }}
+	{{- println }}
+	{{- $checkFailed := false }}
+	{{- range $cf := $f.Checks }}
+		{{- println " " (yellow "--") "checks before migrating version" (cyan $f.File.Version) }}
+		{{- range $s := $cf.Stmts }}
+			{{- if $s.Error }}
+				{{- println "   " (red "->") (indent_ln $s.Stmt 7) }}
+			{{- else }}
+				{{- println "   " (cyan "->") (indent_ln $s.Stmt 7) }}
+			{{- end }}
+		{{- end }}
+		{{- with $cf.Error }}
+			{{- $checkFailed = true }}
+			{{- println "   " (redBgWhiteFg .Text) }}
+		{{- else }}
+			{{- printf "  %s ok (%s)\n\n" (yellow "--") (yellow ($cf.End.Sub $cf.Start).String) }}
+		{{- end }}
+	{{- end }}
+	{{- if $checkFailed }}
+		{{- continue }} {{- /* No statements were applied. */}}
+	{{- end }}
+	{{- println " " (yellow "--") "migrating version" (cyan $f.File.Version) }}
+	{{- range $f.Applied }}
+		{{- println "   " (cyan "->") (indent_ln . 7) }}
+	{{- end }}
+	{{- with .Error }}
+		{{- println "   " (redBgWhiteFg .Text) }}
+	{{- else }}
+		{{- printf "  %s ok (%s)\n" (yellow "--") (yellow (.End.Sub .Start).String) }}
+	{{- end }}
 {{- end }}
+{{- println }}
+{{- println " " (cyan "-------------------------") }}
+{{- println " " (.Summary "  ") }}
+{{- end -}}
 `))
 )
 
@@ -345,8 +336,9 @@ type (
 		migrate.File
 		Start   time.Time
 		End     time.Time
-		Skipped int      // Amount of skipped SQL statements in a partially applied file.
-		Applied []string // SQL statements applied with success
+		Skipped int           // Amount of skipped SQL statements in a partially applied file.
+		Applied []string      // SQL statements applied with success
+		Checks  []*FileChecks // Assertion checks
 		Error   *StmtError
 	}
 )
@@ -382,6 +374,34 @@ func (a *MigrateApply) Log(e migrate.LogEntry) {
 			Start:   time.Now(),
 			Skipped: e.Skip,
 		})
+	case migrate.LogChecks:
+		f := a.Applied[len(a.Applied)-1]
+		f.Checks = append(f.Checks, &FileChecks{
+			Name:  e.Name,
+			Start: time.Now(),
+			Stmts: make([]*Check, 0, len(e.Stmts)),
+		})
+	case migrate.LogCheck:
+		var (
+			f  = a.Applied[len(a.Applied)-1]
+			cf = f.Checks[len(f.Checks)-1]
+			ck = &Check{Stmt: e.Stmt}
+		)
+		if e.Error != nil {
+			m := e.Error.Error()
+			ck.Error = &m
+		}
+		cf.Stmts = append(cf.Stmts, ck)
+	case migrate.LogChecksDone:
+		f := a.Applied[len(a.Applied)-1]
+		cf := f.Checks[len(f.Checks)-1]
+		cf.End = time.Now()
+		if e.Error != nil {
+			cf.Error = &StmtError{
+				Text: e.Error.Error(),
+				Stmt: cf.Stmts[len(cf.Stmts)-1].Stmt,
+			}
+		}
 	case migrate.LogStmt:
 		f := a.Applied[len(a.Applied)-1]
 		f.Applied = append(f.Applied, e.SQL)
@@ -404,10 +424,83 @@ func (a *MigrateApply) Log(e migrate.LogEntry) {
 	}
 }
 
-// CountStmts returns the amount of applied statements.
-func (a *MigrateApply) CountStmts() (n int) {
+// Summary returns a footer of the migration attempt.
+func (a *MigrateApply) Summary(ident string) string {
+	var (
+		passedC, failedC int
+		passedS, failedS int
+		passedF, failedF int
+		lines            = make([]string, 0, 3)
+	)
 	for _, f := range a.Applied {
-		n += len(f.Applied)
+		// For each check file, count the
+		// number of failed assertions.
+		for _, cf := range f.Checks {
+			for _, s := range cf.Stmts {
+				if s.Error != nil {
+					failedC++
+				} else {
+					passedC++
+				}
+			}
+		}
+		passedS += len(f.Applied)
+		if f.Error != nil {
+			failedF++
+			// Last statement failed (not an assertion).
+			if len(f.Checks) == 0 || f.Checks[len(f.Checks)-1].Error == nil {
+				passedS--
+				failedS++
+			}
+		} else {
+			passedF++
+		}
+	}
+	// Execution time.
+	lines = append(lines, fmt.Sprintf(a.End.Sub(a.Start).String()))
+	// Executed files.
+	switch {
+	case passedF > 0 && failedF > 0:
+		lines = append(lines, fmt.Sprintf("%d migration%s ok, %d with errors", passedF, plural(passedF), failedF))
+	case passedF > 0:
+		lines = append(lines, fmt.Sprintf("%d migration%s", passedF, plural(passedF)))
+	case failedF > 0:
+		lines = append(lines, fmt.Sprintf("%d migration%s with errors", failedF, plural(failedF)))
+	}
+	// Executed checks.
+	switch {
+	case passedC > 0 && failedC > 0:
+		lines = append(lines, fmt.Sprintf("%d check%s ok, %d failure%s", passedC, plural(passedC), failedC, plural(failedC)))
+	case passedC > 0:
+		lines = append(lines, fmt.Sprintf("%d check%s", passedC, plural(passedC)))
+	case failedC > 0:
+		lines = append(lines, fmt.Sprintf("%d check error%s", failedC, plural(failedC)))
+	}
+	// Executed statements.
+	switch {
+	case passedS > 0 && failedS > 0:
+		lines = append(lines, fmt.Sprintf("%d sql statement%s ok, %d with errors", passedS, plural(passedS), failedS))
+	case passedS > 0:
+		lines = append(lines, fmt.Sprintf("%d sql statement%s", passedS, plural(passedS)))
+	case failedS > 0:
+		lines = append(lines, fmt.Sprintf("%d sql statement%s with errors", failedS, plural(failedS)))
+	}
+	var b strings.Builder
+	for i, l := range lines {
+		b.WriteString(ColorYellow("--"))
+		b.WriteByte(' ')
+		b.WriteString(l)
+		if i < len(lines)-1 {
+			b.WriteByte('\n')
+			b.WriteString(ident)
+		}
+	}
+	return b.String()
+}
+
+func plural(n int) (s string) {
+	if n > 1 {
+		s += "s"
 	}
 	return
 }
@@ -534,8 +627,9 @@ type SchemaInspect struct {
 var (
 	// InspectTemplateFuncs are global functions available in inspect report templates.
 	InspectTemplateFuncs = template.FuncMap{
-		"sql":  sqlInspect,
-		"json": jsonEncode,
+		"sql":     sqlInspect,
+		"json":    jsonEncode,
+		"mermaid": mermaid,
 	}
 
 	// SchemaInspectTemplate holds the default template of the 'schema inspect' command.
@@ -690,14 +784,20 @@ func sqlInspect(report *SchemaInspect, indent ...string) (string, error) {
 		if report.Client.URL.Schema == "" {
 			changes = append(changes, &schema.AddSchema{S: s})
 		}
+		for _, o := range s.Objects {
+			changes = append(changes, &schema.AddObject{O: o})
+		}
 		for _, t := range s.Tables {
 			changes = append(changes, &schema.AddTable{T: t})
+			for _, r := range t.Triggers {
+				changes = append(changes, &schema.AddTrigger{T: r})
+			}
 		}
 		for _, v := range s.Views {
 			changes = append(changes, &schema.AddView{V: v})
-		}
-		for _, o := range s.Objects {
-			changes = append(changes, &schema.AddObject{O: o})
+			for _, r := range v.Triggers {
+				changes = append(changes, &schema.AddTrigger{T: r})
+			}
 		}
 		for _, f := range s.Funcs {
 			changes = append(changes, &schema.AddFunc{F: f})
@@ -787,6 +887,89 @@ func merge(maps ...template.FuncMap) template.FuncMap {
 	}
 }
 
+func mermaid(i *SchemaInspect, _ ...string) (string, error) {
+	ft, ok := i.Driver.(interface {
+		FormatType(schema.Type) (string, error)
+	})
+	if !ok {
+		return "", fmt.Errorf("mermaid: driver does not support FormatType")
+	}
+	var (
+		b       strings.Builder
+		qualify = len(i.Realm.Schemas) > 1
+		funcs   = template.FuncMap{
+			"nospace":    strings.NewReplacer(" ", "_").Replace,
+			"formatType": ft.FormatType,
+			"tableName": func(t *schema.Table) string {
+				if qualify {
+					return fmt.Sprintf("%[1]s_%[2]s[\"%[1]s.%[2]s\"]", t.Schema.Name, t.Name)
+				}
+				return t.Name
+			},
+			"tableIdent": func(t *schema.Table) string {
+				if qualify {
+					return fmt.Sprintf("%s_%s", t.Schema.Name, t.Name)
+				}
+				return t.Name
+			},
+			"pkfk": func(t *schema.Table, c *schema.Column) string {
+				var pkfk []string
+				if t.PrimaryKey != nil && slices.ContainsFunc(t.PrimaryKey.Parts, func(p *schema.IndexPart) bool { return p.C == c }) {
+					pkfk = append(pkfk, "PK")
+				}
+				if c.ForeignKeys != nil {
+					pkfk = append(pkfk, "FK")
+				}
+				return strings.Join(pkfk, ",")
+			},
+			"card": func(fk *schema.ForeignKey) string {
+				var (
+					hasU = func(t *schema.Table, cs []*schema.Column) bool {
+						if t.PrimaryKey != nil && slices.EqualFunc(t.PrimaryKey.Parts, cs, func(p *schema.IndexPart, c *schema.Column) bool {
+							return p.C != nil && p.C.Name == c.Name
+						}) {
+							return true
+						}
+						return slices.ContainsFunc(t.Indexes, func(idx *schema.Index) bool {
+							return idx.Unique && slices.EqualFunc(idx.Parts, cs, func(p *schema.IndexPart, c *schema.Column) bool {
+								return p.C != nil && p.C.Name == c.Name
+							})
+						})
+					}
+					from, to = "}", "{"
+				)
+				if hasU(fk.Table, fk.Columns) {
+					from = "|"
+				}
+				if hasU(fk.RefTable, fk.RefColumns) {
+					to = "|"
+				}
+				return fmt.Sprintf("%so--o%s", from, to)
+			},
+		}
+		t = template.Must(template.New("mermaid").
+			Funcs(funcs).
+			Parse(`erDiagram
+{{- range $s := .Schemas }}
+  {{- range $t := $s.Tables }}
+    {{ tableName $t }} {
+    {{- range $c := $t.Columns }}
+      {{ formatType $c.Type.Type | nospace }} {{ nospace $c.Name }}{{ with pkfk $t $c }} {{ . }}{{ end }}
+    {{- end }}
+    }
+    {{- range $fk := $t.ForeignKeys }}
+    {{ tableIdent $t }} {{ card $fk }} {{ tableIdent $fk.RefTable }} : {{ $fk.Symbol }}
+    {{- end }}
+  {{- end }}
+{{- end }}
+`))
+	)
+	if err := t.Execute(&b, i.Realm); err != nil {
+		return "", err
+	}
+	return b.String(), nil
+}
+
 func jsonEncode(v any, args ...string) (string, error) {
 	var (
 		b   []byte
@@ -817,6 +1000,11 @@ func jsonMerge(objects ...string) (string, error) {
 	return string(b), nil
 }
 
-func dec(i int) int {
-	return i - 1
+func add(a, b int) int {
+	return a + b
+}
+
+func indentLn(input string, indent int) string {
+	pad := strings.Repeat(" ", indent)
+	return strings.ReplaceAll(input, "\n", "\n"+pad)
 }
