@@ -25,12 +25,16 @@ import (
 	"github.com/go-sql-driver/mysql"
 )
 
-const pass = "pass"
+const (
+	pass          = "pass"
+	passSQLServer = "P@ssw0rd0995"
+)
 
 type (
 	// Config is used to configure container creation.
 	Config struct {
-		setup []string // contains statements to execute once the service is up
+		driver string   // driver to open connections with.
+		setup  []string // contains statements to execute once the service is up
 		// Image is the name of the image to pull and run.
 		Image string
 		// Env vars to pass to the docker container.
@@ -68,43 +72,123 @@ func NewConfig(opts ...ConfigOption) (*Config, error) {
 	return c, nil
 }
 
+const (
+	DriverMySQL      = "mysql"
+	DriverMariaDB    = "mariadb"
+	DriverPostgres   = "postgres"
+	DriverSQLServer  = "sqlserver"
+	DriverClickHouse = "clickhouse"
+)
+
 // FromURL parses a URL in the format of
-// "docker://image/tag" and returns a Config.
+// "docker://driver/tag[/dbname]" and returns a Config.
 func FromURL(u *url.URL) (*Config, error) {
 	var (
-		tag   string
-		opts  []ConfigOption
-		parts = strings.Split(strings.TrimPrefix(u.Path, "/"), "/")
+		parts  = strings.Split(strings.TrimPrefix(u.Path, "/"), "/")
+		idxTag = len(parts) - 1
+		dbName string
 	)
-	if len(parts) > 0 {
-		tag = parts[0]
+	// Check if the last part is a tag or a database name.
+	if idxTag > 0 && !strings.ContainsRune(parts[idxTag], ':') {
+		// The last part is not a tag, so it must be a database name.
+		dbName, idxTag = parts[idxTag], idxTag-1
 	}
-	if len(parts) > 1 {
-		opts = append(opts, Database(parts[1]))
+	var (
+		opts []ConfigOption
+		tag  string
+	)
+	// Support docker+driver://<image>[:<tag>]
+	driver, customImage := strings.CutPrefix(u.Scheme, "docker+")
+	if customImage {
+		// The image is fully specified in the URL.
+		img := path.Join(parts[:idxTag+1]...)
+		if u.Host != "" && u.Host != "_" {
+			img = path.Join(u.Host, img)
+		}
+		opts = append(opts, Image(img))
+	} else {
+		driver = u.Host
+		if idxTag >= 0 {
+			tag = parts[idxTag]
+		}
 	}
-	switch u.Host {
-	case "mysql":
-		if len(parts) > 1 {
-			opts = append(opts, Env("MYSQL_DATABASE="+parts[1]), setup(fmt.Sprintf("CREATE DATABASE IF NOT EXISTS `%s`", parts[1])))
+	var (
+		err error
+		cfg *Config
+	)
+	switch driver {
+	case DriverMySQL:
+		if dbName != "" {
+			opts = append(opts,
+				Database(dbName),
+				Env("MYSQL_DATABASE="+dbName),
+				Setup(fmt.Sprintf("CREATE DATABASE IF NOT EXISTS `%s`", dbName)),
+			)
 		}
-		return MySQL(tag, opts...)
-	case "maria", "mariadb":
-		if len(parts) > 1 {
-			opts = append(opts, Env("MYSQL_DATABASE="+parts[1]), setup(fmt.Sprintf("CREATE DATABASE IF NOT EXISTS `%s`", parts[1])))
+		cfg, err = MySQL(tag, opts...)
+	case "maria":
+		driver = DriverMariaDB
+		fallthrough
+	case DriverMariaDB:
+		if dbName != "" {
+			opts = append(opts,
+				Database(dbName),
+				Env("MYSQL_DATABASE="+dbName),
+				Setup(fmt.Sprintf("CREATE DATABASE IF NOT EXISTS `%s`", dbName)),
+			)
 		}
-		return MariaDB(tag, opts...)
-	case "postgres":
-		if len(parts) > 1 {
-			opts = append(opts, Env("POSTGRES_DB="+parts[1]))
-		}
-		return PostgreSQL(tag, opts...)
+		cfg, err = MariaDB(tag, opts...)
 	case "postgis":
-		if len(parts) > 1 {
-			opts = append(opts, Env("POSTGRES_DB="+parts[1]))
+		opts = append(opts, Image("postgis/postgis:"+tag))
+		driver = DriverPostgres
+		fallthrough
+	case DriverPostgres:
+		if dbName != "" {
+			opts = append(opts, Database(dbName), Env("POSTGRES_DB="+dbName))
 		}
-		return Postgis(tag, opts...)
+		cfg, err = PostgreSQL(tag, opts...)
+	case DriverSQLServer:
+		if dbName != "" && dbName != "master" {
+			opts = append(opts,
+				Database(dbName),
+				Setup(fmt.Sprintf("CREATE DATABASE [%s]", dbName)),
+			)
+		}
+		cfg, err = SQLServer(tag, opts...)
+	case DriverClickHouse:
+		if dbName != "" {
+			opts = append(opts,
+				Database(dbName),
+				Env("CLICKHOUSE_DB="+dbName),
+				Setup(fmt.Sprintf("CREATE DATABASE IF NOT EXISTS `%s`", dbName)),
+			)
+		}
+		cfg, err = ClickHouse(tag, opts...)
 	default:
-		return nil, fmt.Errorf("unsupported docker image %q", u.Host)
+		return nil, fmt.Errorf("unsupported docker image %q", driver)
+	}
+	if err != nil {
+		return nil, err
+	}
+	cfg.driver = driver
+	return cfg, nil
+}
+
+// ImageURL returns the base URL for the given driver and image.
+func ImageURL(driver string, image string) (*url.URL, error) {
+	switch {
+	case driver == "" && image == "":
+		return nil, errors.New("driver and image cannot be empty")
+	case driver == "":
+		return nil, errors.New("driver cannot be empty")
+	case image == "":
+		return nil, errors.New("image cannot be empty")
+	default:
+		u := &url.URL{Scheme: "docker+" + driver, Host: "_", Path: image}
+		if idx := strings.IndexByte(image, '/'); idx != -1 {
+			u.Host, u.Path = image[:idx], image[idx+1:]
+		}
+		return u, nil
 	}
 }
 
@@ -146,15 +230,33 @@ func PostgreSQL(version string, opts ...ConfigOption) (*Config, error) {
 	)
 }
 
-// Postgis returns a new Config for a Postgis image. This is a Postgres variant
-func Postgis(version string, opts ...ConfigOption) (*Config, error) {
+// SQLServer returns a new Config for a SQLServer image.
+func SQLServer(version string, opts ...ConfigOption) (*Config, error) {
 	return NewConfig(
 		append(
 			[]ConfigOption{
-				Image("postgis/postgis:" + version),
-				Port("5432"),
-				Database("postgres"),
-				Env("POSTGRES_PASSWORD=" + pass),
+				Image("mcr.microsoft.com/mssql/server:" + version),
+				Port("1433"),
+				Database("master"),
+				Env(
+					"ACCEPT_EULA=Y",
+					"MSSQL_PID=Developer",
+					"MSSQL_SA_PASSWORD="+passSQLServer,
+				),
+			},
+			opts...,
+		)...,
+	)
+}
+
+// ClickHouse returns a new Config for a ClickHouse image.
+func ClickHouse(version string, opts ...ConfigOption) (*Config, error) {
+	return NewConfig(
+		append(
+			[]ConfigOption{
+				Image("clickhouse/clickhouse-server:" + version),
+				Port("9000"),
+				Env("CLICKHOUSE_PASSWORD=" + pass),
 			},
 			opts...,
 		)...,
@@ -217,11 +319,11 @@ func Out(w io.Writer) ConfigOption {
 	}
 }
 
-// setup adds statements to execute once the service is ready. For example:
+// Setup adds statements to execute once the service is ready. For example:
 //
 //	setup("CREATE DATABASE IF NOT EXISTS test")
 //	setup("DROP SCHEMA IF EXISTS public CASCADE")
-func setup(s ...string) ConfigOption {
+func Setup(s ...string) ConfigOption {
 	return func(c *Config) error {
 		c.setup = append(c.setup, s...)
 		return nil
@@ -239,11 +341,6 @@ func (c *Config) Run(ctx context.Context) (*Container, error) {
 	if err != nil {
 		return nil, fmt.Errorf("getting open port: %w", err)
 	}
-	// Make sure the image is up-to-date.
-	out, err := exec.CommandContext(ctx, "docker", "pull", c.Image).CombinedOutput() //nolint:gosec
-	if err != nil {
-		return nil, fmt.Errorf("pulling image: %w: %q", err, out)
-	}
 	// Run the container.
 	args := []string{"docker", "run", "--rm", "--detach"}
 	for _, e := range c.Env {
@@ -251,14 +348,17 @@ func (c *Config) Run(ctx context.Context) (*Container, error) {
 	}
 	args = append(args, "-p", fmt.Sprintf("%s:%s", p, c.Port), c.Image)
 	cmd := exec.CommandContext(ctx, args[0], args[1:]...) //nolint:gosec
-	buf := &bytes.Buffer{}
-	cmd.Stdout = io.MultiWriter(c.Out, buf)
+	stdout, stderr := &bytes.Buffer{}, &bytes.Buffer{}
+	cmd.Stdout, cmd.Stderr = io.MultiWriter(c.Out, stdout), stderr
 	if err := cmd.Run(); err != nil {
+		if stderr.Len() > 0 {
+			err = errors.New(strings.TrimSpace(stderr.String()))
+		}
 		return nil, err
 	}
 	return &Container{
 		cfg:        *c,
-		ID:         strings.TrimSpace(buf.String()),
+		ID:         strings.TrimSpace(stdout.String()),
 		Passphrase: pass,
 		Port:       p,
 		out:        c.Out,
@@ -285,12 +385,12 @@ func (c *Container) Wait(ctx context.Context, timeout time.Duration) error {
 	if err != nil {
 		return err
 	}
+	pingURL := c.PingURL(*u)
 	for {
 		select {
 		case <-time.After(100 * time.Millisecond):
 			// Ping against the root connection.
-			u.Path = "/"
-			client, err := sqlclient.Open(ctx, u.String())
+			client, err := sqlclient.Open(ctx, pingURL)
 			if err != nil {
 				continue
 			}
@@ -317,13 +417,31 @@ func (c *Container) Wait(ctx context.Context, timeout time.Duration) error {
 
 // URL returns a URL to connect to the Container.
 func (c *Container) URL() (*url.URL, error) {
-	switch img := path.Base(strings.SplitN(c.cfg.Image, ":", 2)[0]); img {
-	case "postgres", "postgis":
+	switch c.cfg.driver {
+	case DriverClickHouse:
+		return url.Parse(fmt.Sprintf("clickhouse://:%s@%s:%s/%s", c.Passphrase, "localhost", c.Port, c.cfg.Database))
+	case DriverSQLServer:
+		return url.Parse(fmt.Sprintf("sqlserver://sa:%s@localhost:%s?database=%s", passSQLServer, c.Port, c.cfg.Database))
+	case DriverPostgres:
 		return url.Parse(fmt.Sprintf("postgres://postgres:%s@localhost:%s/%s?sslmode=disable", c.Passphrase, c.Port, c.cfg.Database))
-	case "mysql", "mariadb":
-		return url.Parse(fmt.Sprintf("%s://root:%s@localhost:%s/%s", img, c.Passphrase, c.Port, c.cfg.Database))
+	case DriverMySQL, DriverMariaDB:
+		return url.Parse(fmt.Sprintf("%s://root:%s@localhost:%s/%s", c.cfg.driver, c.Passphrase, c.Port, c.cfg.Database))
 	default:
-		return nil, fmt.Errorf("unknown container image: %q", img)
+		return nil, fmt.Errorf("unknown driver: %q", c.cfg.driver)
+	}
+}
+
+// PingURL returns a URL to ping the Container.
+func (c *Container) PingURL(u url.URL) string {
+	switch c.cfg.driver {
+	case DriverSQLServer:
+		q := u.Query()
+		q.Del("database")
+		u.RawQuery = q.Encode()
+		return u.String()
+	default:
+		u.Path = "/"
+		return u.String()
 	}
 }
 
@@ -351,10 +469,18 @@ func freePort() (string, error) {
 }
 
 func init() {
-	sqlclient.Register("docker", sqlclient.OpenerFunc(client))
+	sqlclient.Register(
+		"docker",
+		sqlclient.OpenerFunc(Open),
+		sqlclient.RegisterFlavours(
+			"docker+postgres", "docker+mysql", "docker+maria",
+			"docker+sqlserver", "docker+clickhouse",
+		),
+	)
 }
 
-func client(ctx context.Context, u *url.URL) (client *sqlclient.Client, err error) {
+// Open a new docker client.
+func Open(ctx context.Context, u *url.URL) (client *sqlclient.Client, err error) {
 	cfg, err := FromURL(u)
 	if err != nil {
 		return nil, err
