@@ -2,6 +2,8 @@
 // This source code is licensed under the Apache 2.0 license found
 // in the LICENSE file in the root directory of this source tree.
 
+//go:build !windows
+
 package pgparse
 
 import (
@@ -20,8 +22,8 @@ import (
 type Parser struct{}
 
 // ColumnFilledBefore checks if the column was filled before the given position.
-func (p *Parser) ColumnFilledBefore(f migrate.File, t *schema.Table, c *schema.Column, pos int) (bool, error) {
-	return parseutil.MatchStmtBefore(f, pos, func(s *migrate.Stmt) (bool, error) {
+func (p *Parser) ColumnFilledBefore(stmts []*migrate.Stmt, t *schema.Table, c *schema.Column, pos int) (bool, error) {
+	return parseutil.MatchStmtBefore(stmts, pos, func(s *migrate.Stmt) (bool, error) {
 		tr, err := pgquery.Parse(s.Text)
 		if err != nil {
 			return false, err
@@ -59,8 +61,8 @@ func (p *Parser) ColumnFilledBefore(f migrate.File, t *schema.Table, c *schema.C
 }
 
 // CreateViewAfter checks if a view was created after the position with the given name to a table.
-func (p *Parser) CreateViewAfter(f migrate.File, old, new string, pos int) (bool, error) {
-	return parseutil.MatchStmtAfter(f, pos, func(s *migrate.Stmt) (bool, error) {
+func (p *Parser) CreateViewAfter(stmts []*migrate.Stmt, old, new string, pos int) (bool, error) {
+	return parseutil.MatchStmtAfter(stmts, pos, func(s *migrate.Stmt) (bool, error) {
 		tr, err := pgquery.Parse(s.Text)
 		if err != nil {
 			return false, err
@@ -76,7 +78,7 @@ func (p *Parser) CreateViewAfter(f migrate.File, old, new string, pos int) (bool
 			return false, nil
 		}
 		from := v.Query.GetSelectStmt().GetFromClause()
-		if f == nil || len(from) != 1 {
+		if len(from) != 1 {
 			return false, nil
 		}
 		return from[0].GetRangeVar().GetRelname() == new, nil
@@ -138,6 +140,29 @@ func (p *Parser) FixChange(_ migrate.Driver, s string, changes schema.Changes) (
 			}) {
 				add.Extra = append(add.Extra, &postgres.Concurrently{})
 			}
+		case stmt.GetDropStmt() != nil && stmt.GetDropStmt().GetConcurrent():
+			modify, err := expectModify(changes)
+			if err != nil {
+				return nil, err
+			}
+			for _, p := range stmt.GetDropStmt().GetObjects() {
+				items := p.GetList().GetItems()
+				if len(items) != 1 || items[0].GetString_().GetSval() == "" {
+					continue
+				}
+				name := items[0].GetString_().GetSval()
+				i := schema.Changes(modify.Changes).IndexDropIndex(name)
+				if i == -1 {
+					return nil, fmt.Errorf("DropIndex %q command not found", name)
+				}
+				drop := modify.Changes[i].(*schema.DropIndex)
+				if !slices.ContainsFunc(drop.Extra, func(c schema.Clause) bool {
+					_, ok := c.(*postgres.Concurrently)
+					return ok
+				}) {
+					drop.Extra = append(drop.Extra, &postgres.Concurrently{})
+				}
+			}
 		}
 	}
 	return changes, nil
