@@ -204,6 +204,144 @@ func TestFixChange_CreateIndexCon(t *testing.T) {
 	require.Equal(t, &postgres.Concurrently{}, m.Changes[0].(*schema.AddIndex).Extra[0])
 }
 
+func TestFixChange_DropIndexCon(t *testing.T) {
+	var p pgparse.Parser
+	changes, err := p.FixChange(
+		nil,
+		"DROP INDEX i1",
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{I: schema.NewIndex("i1")},
+				},
+			},
+		},
+	)
+	require.NoError(t, err)
+	// No changes.
+	require.Equal(
+		t,
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{
+						I: schema.NewIndex("i1"),
+					},
+				},
+			},
+		},
+		changes,
+	)
+
+	changes, err = p.FixChange(
+		nil,
+		"DROP INDEX CONCURRENTLY i1",
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{I: schema.NewIndex("i1")},
+				},
+			},
+		},
+	)
+	require.NoError(t, err)
+	// Should add the "Concurrently" clause to the DropIndex command.
+	require.Equal(
+		t,
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{
+						I: schema.NewIndex("i1"),
+						Extra: []schema.Clause{
+							&postgres.Concurrently{},
+						},
+					},
+				},
+			},
+		},
+		changes,
+	)
+
+	changes, err = p.FixChange(
+		nil,
+		"DROP INDEX CONCURRENTLY i1",
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{
+						I: schema.NewIndex("i1"),
+						Extra: []schema.Clause{
+							&postgres.Concurrently{},
+						},
+					},
+				},
+			},
+		},
+	)
+	require.NoError(t, err)
+	// The "Concurrently" clause should not be added if it already exists.
+	require.Equal(
+		t,
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{
+						I: schema.NewIndex("i1"),
+						Extra: []schema.Clause{
+							&postgres.Concurrently{},
+						},
+					},
+				},
+			},
+		},
+		changes,
+	)
+	// Support quoted identifiers.
+	changes, err = p.FixChange(
+		nil,
+		`DROP INDEX CONCURRENTLY "i1"`,
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{I: schema.NewIndex("i1")},
+				},
+			},
+		},
+	)
+	require.NoError(t, err)
+	m, ok := changes[0].(*schema.ModifyTable)
+	require.True(t, ok)
+	require.Equal(t, &postgres.Concurrently{}, m.Changes[0].(*schema.DropIndex).Extra[0])
+
+	// Multiple indexes.
+	changes, err = p.FixChange(
+		nil,
+		`DROP INDEX CONCURRENTLY i1, i2`,
+		schema.Changes{
+			&schema.ModifyTable{
+				T: schema.NewTable("t1"),
+				Changes: schema.Changes{
+					&schema.DropIndex{I: schema.NewIndex("i1")},
+					&schema.DropIndex{I: schema.NewIndex("i2")},
+				},
+			},
+		},
+	)
+	require.NoError(t, err)
+	m, ok = changes[0].(*schema.ModifyTable)
+	require.True(t, ok)
+	require.Equal(t, &postgres.Concurrently{}, m.Changes[0].(*schema.DropIndex).Extra[0])
+	require.Equal(t, &postgres.Concurrently{}, m.Changes[1].(*schema.DropIndex).Extra[0])
+}
+
 func TestFixChange_RenameTable(t *testing.T) {
 	var p pgparse.Parser
 	changes, err := p.FixChange(
@@ -281,11 +419,10 @@ func TestColumnFilledBefore(t *testing.T) {
 		},
 	} {
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
-			var (
-				p pgparse.Parser
-				f = migrate.NewLocalFile("file", []byte(tt.file))
-			)
-			filled, err := p.ColumnFilledBefore(f, schema.NewTable("t"), schema.NewColumn("c"), tt.pos)
+			var p pgparse.Parser
+			stmts, err := migrate.Stmts(tt.file)
+			require.NoError(t, err)
+			filled, err := p.ColumnFilledBefore(stmts, schema.NewTable("t"), schema.NewColumn("c"), tt.pos)
 			require.Equal(t, err != nil, tt.wantErr, err)
 			require.Equal(t, filled, tt.wantFilled)
 		})
@@ -345,11 +482,10 @@ CREATE VIEW old AS SELECT * FROM new;
 		},
 	} {
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
-			var (
-				p pgparse.Parser
-				f = migrate.NewLocalFile("file", []byte(tt.file))
-			)
-			created, err := p.CreateViewAfter(f, "old", "new", tt.pos)
+			var p pgparse.Parser
+			stmts, err := migrate.Stmts(tt.file)
+			require.NoError(t, err)
+			created, err := p.CreateViewAfter(stmts, "old", "new", tt.pos)
 			require.Equal(t, err != nil, tt.wantErr, err)
 			require.Equal(t, tt.wantCreated, created)
 		})
