@@ -1032,50 +1032,6 @@ DROP TABLE "posts";`, diff("test2?sslmode=disable&search_path=other", "test2?ssl
 	})
 }
 
-func TestPostgres_MigrateDiffEnt(t *testing.T) {
-	bin := cliPath(t)
-	pgRun(t, func(t *pgTest) {
-		dir := t.TempDir()
-		_, err := t.db.Exec("CREATE SCHEMA IF NOT EXISTS entdev")
-		require.NoError(t, err)
-		t.dropSchemas("entdev")
-
-		entdir, migratedir := filepath.Join(dir, "entschema"), filepath.Join(dir, "migrations")
-		require.NoError(t, os.Mkdir(entdir, 0755))
-		err = os.WriteFile(filepath.Join(entdir, "user.go"), []byte(`
-package schema
-
-import "entgo.io/ent"
-
-type User struct { ent.Schema }
-`), 0600)
-		require.NoError(t, err)
-		for _, ts := range [][]string{{"init", "entschema"}, {"tidy"}} {
-			cmd := exec.Command("go", append([]string{"mod"}, ts...)...)
-			cmd.Dir = dir
-			require.NoError(t, cmd.Run())
-		}
-		cmd := exec.Command(
-			bin, "migrate", "diff",
-			"--dev-url", fmt.Sprintf("postgres://postgres:pass@localhost:%d/test?search_path=entdev&sslmode=disable", t.port),
-			"--to", "ent://"+entdir,
-			"--dir", "file://"+migratedir,
-		)
-		cmd.Dir = dir
-		require.NoError(t, cmd.Run())
-		local, err := migrate.NewLocalDir(migratedir)
-		require.NoError(t, err)
-		files, err := local.Files()
-		require.NoError(t, err)
-		require.Equal(t, 1, len(files))
-		stmts, err := files[0].Stmts()
-		require.NoError(t, err)
-		require.Equal(t, []string{
-			`CREATE TABLE "users" ("id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY, PRIMARY KEY ("id"));`,
-		}, stmts)
-	})
-}
-
 func TestPostgres_DefaultsHCL(t *testing.T) {
 	n := "atlas_defaults"
 	pgRun(t, func(t *pgTest) {
@@ -1178,8 +1134,11 @@ create table atlas_types_sanity
 		require.Len(t, realm.Schemas, 1)
 		ts, ok := realm.Schemas[0].Table(n)
 		require.True(t, ok)
+		require.Len(t, ts.Attrs, 1)
+		require.IsType(t, &postgres.OID{}, ts.Attrs[0])
 		expected := schema.Table{
 			Name:   n,
+			Attrs:  ts.Attrs,
 			Schema: realm.Schemas[0],
 			Columns: []*schema.Column{
 				{
