@@ -6,7 +6,6 @@ package schemahcl
 
 import (
 	"bytes"
-	"errors"
 	"fmt"
 	"reflect"
 	"sort"
@@ -29,8 +28,9 @@ type (
 		funcs            map[string]function.Function
 		pathVars         map[string]map[string]cty.Value
 		pathFuncs        map[string]map[string]function.Function
-		datasrc, initblk map[string]func(*hcl.EvalContext, *hclsyntax.Block) (cty.Value, error)
 		validator        func() SchemaValidator
+		datasrc, initblk map[string]func(*hcl.EvalContext, *hclsyntax.Block) (cty.Value, error)
+		typedblk         map[string]map[string]func(*hcl.EvalContext, *hclsyntax.Block) (cty.Value, error)
 	}
 	// Option configures a Config.
 	Option func(*Config)
@@ -137,6 +137,37 @@ func WithDataSource(name string, h func(*hcl.EvalContext, *hclsyntax.Block) (cty
 	}
 }
 
+// WithTypeLabelBlock registers a type-block and its label along with the corresponding handler.
+// e.g., the example below registers a typed block named "driver" with the label "remote" that
+// returns the string defined in the token attribute.
+//
+//	WithTypeLabelBlock("driver", "remote", func(ctx *hcl.EvalContext, b *hclsyntax.Block) (cty.Value, hcl.Diagnostics) {
+//		attrs, diags := b.Body.JustAttributes()
+//		if diags.HasErrors() {
+//			return cty.NilVal, diags
+//		}
+//		v, diags := attrs["token"].Expr.Value(ctx)
+//		if diags.HasErrors() {
+//			return cty.NilVal, diags
+//		}
+//		return cty.ObjectVal(map[string]cty.Value{"url": v}), nil
+//	})
+//
+//	driver "remote" "hello" {
+//	  token = "hello world"
+//	}
+func WithTypeLabelBlock(name, label string, h func(*hcl.EvalContext, *hclsyntax.Block) (cty.Value, error)) Option {
+	return func(c *Config) {
+		if c.typedblk == nil {
+			c.typedblk = make(map[string]map[string]func(*hcl.EvalContext, *hclsyntax.Block) (cty.Value, error))
+		}
+		if c.typedblk[name] == nil {
+			c.typedblk[name] = make(map[string]func(*hcl.EvalContext, *hclsyntax.Block) (cty.Value, error))
+		}
+		c.typedblk[name][label] = h
+	}
+}
+
 // WithInitBlock registers a block that evaluates (first) to a cty.Value,
 // has no labels, and can be defined only once. For example:
 //
@@ -263,14 +294,20 @@ func (s *State) EvalFiles(paths []string, v any, input map[string]cty.Value) err
 // Eval evaluates the parsed HCL documents using the input variables and populates v
 // using the result.
 func (s *State) Eval(parsed *hclparse.Parser, v any, input map[string]cty.Value) error {
-	ctx := s.newCtx()
-	reg := &blockDef{
-		fields:   make(map[string]struct{}),
-		children: make(map[string]*blockDef),
+	var (
+		ctx          = s.newCtx()
+		files        = parsed.Files()
+		fileNames    = make([]string, 0, len(files))
+		metaBlocks   = make(map[string][]*hclsyntax.Block, len(files))
+		staticBlocks = make([]*hclsyntax.Block, 0, len(files))
+		reg          = &blockDef{
+			fields:   make(map[string]struct{}),
+			children: make(map[string]*blockDef),
+		}
+	)
+	if ctx.Variables == nil {
+		ctx.Variables = make(map[string]cty.Value)
 	}
-	files := parsed.Files()
-	fileNames := make([]string, 0, len(files))
-	allBlocks := make([]*hclsyntax.Block, 0, len(files))
 	for name, file := range files {
 		fileNames = append(fileNames, name)
 		if err := s.setInputVals(ctx, file.Body, input); err != nil {
@@ -283,33 +320,56 @@ func (s *State) Eval(parsed *hclparse.Parser, v any, input map[string]cty.Value)
 		blocks := make(hclsyntax.Blocks, 0, len(body.Blocks))
 		for _, b := range body.Blocks {
 			switch {
-			// Variable blocks are not reachable by reference.
 			case b.Type == BlockVariable:
-				continue
-			// Semi-evaluate blocks with the for_each meta argument.
 			case b.Body != nil && b.Body.Attributes[forEachAttr] != nil:
+				metaBlocks[name] = append(metaBlocks[name], b)
+			default:
+				blocks = append(blocks, b)
+				reg.child(extractDef(b, reg))
+			}
+		}
+		body.Blocks = blocks
+		staticBlocks = append(staticBlocks, blocks...)
+	}
+	vars, err := blockVars(staticBlocks, "", reg)
+	if err != nil {
+		return err
+	}
+	for k, v := range vars {
+		ctx.Variables[k] = v
+	}
+	// Semi-evaluate blocks with the for_each meta argument.
+	if len(metaBlocks) > 0 {
+		blocks := make([]*hclsyntax.Block, 0, len(metaBlocks))
+		for name, bs := range metaBlocks {
+			for _, b := range bs {
 				nb, err := forEachBlocks(ctx, b)
 				if err != nil {
 					return err
 				}
+				// Extract the definition of the top-level is enough.
+				reg.child(extractDef(b, reg))
 				blocks = append(blocks, nb...)
-			default:
-				blocks = append(blocks, b)
+				files[name].Body.(*hclsyntax.Body).Blocks = append(files[name].Body.(*hclsyntax.Body).Blocks, nb...)
+			}
+		}
+		if vars, err = blockVars(blocks, "", reg); err != nil {
+			return err
+		}
+		for k, v := range vars {
+			if v.IsNull() {
+				continue
+			}
+			if bs, ok := ctx.Variables[k]; !ok || bs.IsNull() {
+				ctx.Variables[k] = v
+			} else {
+				vs := bs.AsValueMap()
+				for k1, v1 := range v.AsValueMap() {
+					vs[k1] = v1
+				}
+				ctx.Variables[k] = cty.ObjectVal(vs)
 			}
-			reg.child(extractDef(b, reg))
 		}
-		body.Blocks = blocks
-		allBlocks = append(allBlocks, blocks...)
-	}
-	vars, err := blockVars(allBlocks, "", reg)
-	if err != nil {
-		return err
-	}
-	if ctx.Variables == nil {
-		ctx.Variables = make(map[string]cty.Value)
-	}
-	for k, v := range vars {
-		ctx.Variables[k] = v
 	}
 	spec := &Resource{}
 	sort.Slice(fileNames, func(i, j int) bool {
@@ -361,13 +421,13 @@ func patchRefs(spec *Resource) error {
 }
 
 func (r addrRef) patch(resource *Resource) error {
-	cp := r.copy().load(resource, "")
+	r.load(resource, "")
 	for _, attr := range resource.Attrs {
 		if !attr.IsRef() {
 			continue
 		}
 		ref := attr.V.EncapsulatedValue().(*Ref)
-		referenced, ok := cp[ref.V]
+		referenced, ok := r[ref.V]
 		if !ok {
 			return fmt.Errorf("broken reference to %q", ref.V)
 		}
@@ -376,21 +436,13 @@ func (r addrRef) patch(resource *Resource) error {
 		}
 	}
 	for _, ch := range resource.Children {
-		if err := cp.patch(ch); err != nil {
+		if err := r.patch(ch); err != nil {
 			return err
 		}
 	}
 	return nil
 }
 
-func (r addrRef) copy() addrRef {
-	n := make(addrRef)
-	for k, v := range r {
-		n[k] = v
-	}
-	return n
-}
-
 // load the references from the children of the resource.
 func (r addrRef) load(res *Resource, track string) addrRef {
 	unlabeled := 0
@@ -424,7 +476,8 @@ func (s *State) resource(ctx *hcl.EvalContext, vr SchemaValidator, file *hcl.Fil
 		return nil, err
 	}
 	res := &Resource{
-		Attrs: attrs,
+		Attrs:    attrs,
+		Children: make([]*Resource, 0, len(body.Blocks)),
 	}
 	for _, blk := range body.Blocks {
 		// variable blocks may be included in the document but are skipped in unmarshaling.
@@ -456,31 +509,15 @@ func (s *State) mayScopeContext(ctx *hcl.EvalContext, scope []string) *hcl.EvalC
 	if !ok1 && !ok2 {
 		return ctx
 	}
-	nctx := &hcl.EvalContext{
-		Variables: make(map[string]cty.Value),
-		Functions: make(map[string]function.Function),
-	}
-	for n, v := range vars {
-		nctx.Variables[n] = v
-	}
-	for n, f := range funcs {
-		nctx.Functions[n] = f
-	}
-	// A patch from the past. Should be moved
-	// to specific scopes in the future.
-	nctx.Functions["sql"] = rawExprImpl()
-	for p := ctx; p != nil; p = p.Parent() {
-		for k, v := range p.Variables {
-			if isRef(v) {
-				nctx.Variables[k] = v
-			}
-		}
-	}
-	return nctx
+	nctx := ctx.NewChild()
+	// Use the same variables/functions maps to avoid copying per scope, but return a
+	// another child context to prevent writes from different blocks to the same maps.
+	nctx.Variables, nctx.Functions = vars, funcs
+	return nctx.NewChild()
 }
 
 func (s *State) toAttrs(ctx *hcl.EvalContext, vr SchemaValidator, hclAttrs hclsyntax.Attributes, scope []string) ([]*Attr, error) {
-	var attrs []*Attr
+	attrs := make([]*Attr, 0, len(hclAttrs))
 	for _, hclAttr := range hclAttrs {
 		var (
 			scope = append(scope, hclAttr.Name)
@@ -496,6 +533,9 @@ func (s *State) toAttrs(ctx *hcl.EvalContext, vr SchemaValidator, hclAttrs hclsy
 		at := &Attr{K: hclAttr.Name}
 		switch t := value.Type(); {
 		case isRef(value):
+			if !value.Type().HasAttribute("__ref") {
+				return nil, fmt.Errorf("%s: invalid reference used in %s", hclAttr.SrcRange, hclAttr.Name)
+			}
 			at.V = cty.CapsuleVal(ctyRefType, &Ref{V: value.GetAttr("__ref").AsString()})
 		case (t.IsTupleType() || t.IsListType() || t.IsSetType()) && value.LengthInt() > 0:
 			var (
@@ -542,11 +582,30 @@ func (s *State) typeError(diag hcl.Diagnostics, scope []string) error {
 			if d.Summary != "Unknown variable" {
 				continue
 			}
-			if t, ok := s.findTypeSpec(e.Traversal.RootName()); ok && len(t.Attributes) > 0 {
-				d.Detail = fmt.Sprintf("Type %q requires at least 1 argument", t.Name)
-			} else if n := len(scope); n > 1 && (s.config.pathVars[path] != nil || s.config.pathFuncs[path] != nil) {
-				d.Summary = strings.Replace(d.Summary, "variable", fmt.Sprintf("%s.%s", scope[n-2], scope[n-1]), 1)
-				d.Detail = strings.Replace(d.Detail, "variable", scope[n-1], 1)
+			switch root := e.Traversal.RootName(); {
+			case root == RefData, s.config.typedblk[root] != nil:
+				var b strings.Builder
+				b.WriteString(root)
+				for _, t := range e.Traversal[1:] {
+					if v, ok := t.(hcl.TraverseAttr); ok {
+						b.WriteString(".")
+						b.WriteString(v.Name)
+					}
+				}
+				d.Summary = "Unknown data source"
+				if s.config.typedblk[root] != nil {
+					d.Summary = "Unknown block type"
+				}
+				d.Detail = fmt.Sprintf("%s does not exist", b.String())
+			case root == RefLocal:
+				d.Summary = "Unknown local"
+			default:
+				if t, ok := s.findTypeSpec(root); ok && len(t.Attributes) > 0 {
+					d.Detail = fmt.Sprintf("Type %q requires at least 1 argument", t.Name)
+				} else if n := len(scope); n > 1 && (s.config.pathVars[path] != nil || s.config.pathFuncs[path] != nil) {
+					d.Summary = strings.Replace(d.Summary, "variable", fmt.Sprintf("%s.%s", scope[n-2], scope[n-1]), 1)
+					d.Detail = strings.Replace(d.Detail, "variable", scope[n-1], 1)
+				}
 			}
 		}
 	}
@@ -598,6 +657,10 @@ func (s *State) toResource(ctx *hcl.EvalContext, vr SchemaValidator, block *hcls
 	}
 	spec.Attrs = attrs
 	for _, blk := range block.Body.Blocks {
+		ctx, err := setBlockVars(ctx.NewChild(), blk.Body)
+		if err != nil {
+			return nil, err
+		}
 		r, err := s.toResource(ctx, vr, blk, append(scope, blk.Type))
 		if err != nil {
 			return nil, err
@@ -879,7 +942,7 @@ func forEachBlocks(ctx *hcl.EvalContext, b *hclsyntax.Block) ([]*hclsyntax.Block
 	if diags.HasErrors() {
 		return nil, diags
 	}
-	if t := forEach.Type(); !t.IsSetType() && !t.IsObjectType() {
+	if t := forEach.Type(); !t.IsSetType() && !t.IsObjectType() && !t.IsTupleType() {
 		return nil, fmt.Errorf("schemahcl: for_each does not support %s type", t.FriendlyName())
 	}
 	delete(b.Body.Attributes, forEachAttr)
@@ -935,23 +998,6 @@ func (f EvalFunc) Eval(p *hclparse.Parser, i any, input map[string]cty.Value) er
 	return f(p, i, input)
 }
 
-func rawExprImpl() function.Function {
-	return function.New(&function.Spec{
-		Params: []function.Parameter{
-			{Name: "def", Type: cty.String, AllowNull: false},
-		},
-		Type: function.StaticReturnType(ctyRawExpr),
-		Impl: func(args []cty.Value, retType cty.Type) (cty.Value, error) {
-			x := args[0].AsString()
-			if len(x) == 0 {
-				return cty.NilVal, errors.New("empty expression")
-			}
-			t := &RawExpr{X: x}
-			return cty.CapsuleVal(ctyRawExpr, t), nil
-		},
-	})
-}
-
 // typeFuncSpec returns the HCL function for defining the type in the spec.
 func typeFuncSpec(typeSpec *TypeSpec) function.Function {
 	spec := &function.Spec{
