@@ -461,6 +461,78 @@ env "dev" {
 		}),
 	})
 	require.EqualError(t, err, `variable "domains": a number is required`)
+
+	var (
+		// For-each resource depends on other resources.
+		doc1 struct {
+			Schema []*struct {
+				Name string `spec:",name"`
+			} `spec:"schema"`
+			Table []*struct {
+				Name   string `spec:"name,name"`
+				Schema *Ref   `spec:"schema"`
+			} `spec:"table"`
+		}
+		b1 = []byte(`
+schema "s1" {}
+schema "s2" {}
+
+table {
+  for_each = {
+    t1 = schema.s1
+	t2 = schema.s2
+  }
+  name = each.key
+  schema = each.value
+}
+`)
+	)
+	err = New().EvalBytes(b1, &doc1, nil)
+	require.NoError(t, err)
+	buf, err := Marshal.MarshalSpec(&doc1)
+	require.NoError(t, err)
+	require.Equal(t, `schema "s1" {
+}
+schema "s2" {
+}
+table "t1" {
+  schema = schema.s1
+}
+table "t2" {
+  schema = schema.s2
+}
+`, string(buf))
+
+	// Tuple of type any.
+	b1 = []byte(`
+schema "s1" {
+  comment = "schema comment"
+}
+schema "s2" {
+  # object without comment.
+}
+
+table {
+  for_each = [schema.s1, schema.s2]
+  name = each.value.name
+  schema = each.value
+}
+`)
+	err = New().EvalBytes(b1, &doc1, nil)
+	require.NoError(t, err)
+	buf, err = Marshal.MarshalSpec(&doc1)
+	require.NoError(t, err)
+	require.Equal(t, `schema "s1" {
+}
+schema "s2" {
+}
+table "s1" {
+  schema = schema.s1
+}
+table "s2" {
+  schema = schema.s2
+}
+`, string(buf))
 }
 
 func TestDataLocalsRefs(t *testing.T) {
@@ -638,6 +710,11 @@ locals {
 }
 `)
 	require.Error(t, New(opts...).EvalBytes(b, &doc, nil), `cyclic reference to "data.text.a"`)
+
+	b = []byte(`
+out = data.unknown.a.output
+`)
+	require.EqualError(t, New(opts...).EvalBytes(b, &doc, nil), `:2,7-11: Unknown data source; data.unknown.a.output does not exist`)
 }
 
 func TestSkippedDataSrc(t *testing.T) {
@@ -697,6 +774,158 @@ v3 = data.dynamic.evaluated3.v
 `)
 	)
 	require.NoError(t, New(opts...).EvalBytes(b, &v, nil))
+
+	b = []byte(`
+locals {
+  a = data.dynamic.not_skipped1.v
+  b = local.c
+  c = "3"
+  d = data.dynamic.not_skipped3.v
+}
+
+data "dynamic" "not_skipped1" {
+  v = "v2"
+  skip = false
+}
+
+data "dynamic" "not_skipped2" {
+  v = "v${local.b}"
+  skip = false
+}
+
+data "dynamic" "not_skipped3" {
+  v = "v4"
+  skip = false
+}
+
+data "dynamic" "not_skipped4" {
+  v = "v5"
+  skip = false
+}
+
+data "dynamic" "skipped1" {
+  v = local.a
+  skip = true
+}
+
+data "dynamic" "skipped2" {
+  v = "v"
+  skip = true
+}
+
+data "dynamic" "skipped3" {
+  v = data.dynamic.skipped2
+  skip = true
+}
+
+top {
+  a = data.dynamic.not_skipped2.v // "v3".
+  block {
+    block {
+      a1 = local.d // "v4".
+      a2 = data.dynamic.not_skipped4.v // "v5".
+    }
+  }
+}
+
+v2 = local.a
+`)
+	var v1 struct {
+		Top struct {
+			A     string `spec:"a"`
+			Block struct {
+				Block struct {
+					A1 string `spec:"a1"`
+					A2 string `spec:"a2"`
+				} `spec:"block"`
+			} `spec:"block"`
+		} `spec:"top"`
+		V2 string `spec:"v2"`
+	}
+	require.NoError(t, New(opts...).EvalBytes(b, &v1, nil))
+	require.Equal(t, "v2", v1.V2)
+	require.Equal(t, "v3", v1.Top.A)
+	require.Equal(t, "v4", v1.Top.Block.Block.A1)
+	require.Equal(t, "v5", v1.Top.Block.Block.A2)
+}
+
+func TestTypeLabelBlock(t *testing.T) {
+	var (
+		callD, callT int
+		opts         = []Option{
+			WithTypeLabelBlock("driver", "remote", func(ctx *hcl.EvalContext, b *hclsyntax.Block) (cty.Value, error) {
+				attrs, diags := b.Body.JustAttributes()
+				if diags.HasErrors() {
+					return cty.NilVal, diags
+				}
+				v, diags := attrs["name"].Expr.Value(ctx)
+				if diags.HasErrors() {
+					return cty.NilVal, diags
+				}
+				callT++
+				return cty.ObjectVal(map[string]cty.Value{"url": cty.StringVal("driver://" + v.AsString())}), nil
+			}),
+			WithTypeLabelBlock("driver", "not_called", func(ctx *hcl.EvalContext, b *hclsyntax.Block) (cty.Value, error) {
+				t.Fatal("should not be called")
+				return cty.NilVal, nil
+			}),
+			WithDataSource("text", func(ctx *hcl.EvalContext, b *hclsyntax.Block) (cty.Value, error) {
+				attrs, diags := b.Body.JustAttributes()
+				if diags.HasErrors() {
+					return cty.NilVal, diags
+				}
+				v, diags := attrs["value"].Expr.Value(ctx)
+				if diags.HasErrors() {
+					return cty.NilVal, diags
+				}
+				callD++
+				return cty.ObjectVal(map[string]cty.Value{"output": v}), nil
+			}),
+		}
+		doc struct {
+			Values []string `spec:"vs"`
+		}
+		b = []byte(`
+locals {
+  a = "a8m"
+}
+
+data "text" "a" {
+  value = local.a
+}
+
+driver "remote" "myapp" {
+  name = data.text.a.output
+}
+
+vs = [
+  driver.remote.myapp.url,
+  data.text.a.output
+]
+`)
+	)
+	require.NoError(t, New(opts...).EvalBytes(b, &doc, nil))
+	require.Equal(t, []string{"driver://a8m", "a8m"}, doc.Values)
+	require.EqualValues(t, 1, callT)
+	require.EqualValues(t, 2, callD, "it is up to the data source to implement caching")
+
+	b = []byte(`
+locals {
+  a = "a8m"
+}
+
+data "text" "a" {
+  value = local.a
+}
+
+driver "remote" "myapp" {
+  name = data.text.a.output
+}
+`)
+	require.NoError(t, New(opts...).EvalBytes(b, &doc, nil))
+	require.Equal(t, []string{"driver://a8m", "a8m"}, doc.Values)
+	require.Equal(t, 1, callT)
+	require.Equal(t, 2, callD)
 }
 
 type countValidator struct{ nb, na int }
@@ -736,3 +965,95 @@ attr2 = "b"
 	require.Equal(t, 3, cv.nb)
 	require.Equal(t, 2, cv.na)
 }
+
+func Test_ExtraReferences(t *testing.T) {
+	type (
+		Foo struct {
+			Name      string `spec:",name"`
+			Qualifier string `spec:",qualifier"`
+			DefaultExtension
+		}
+		Block1 struct {
+			Name string `spec:",name"`
+			Type string `spec:"type"`
+		}
+		Block2 struct {
+			Name string `spec:",name"`
+			Refs []*Ref `spec:"refs"`
+		}
+		ExtraBar struct {
+			Blk1 *Block1 `spec:"blk1"`
+			Blk2 *Block2 `spec:"blk2"`
+		}
+	)
+	var (
+		doc struct {
+			Foo []*Foo `spec:"foo"`
+		}
+		b = []byte(`
+foo "f1" {
+	extra_bar {
+		blk1 "c1" {
+			type = "c1"
+		}
+		blk2 "p1" {
+			refs = [blk1.c1]
+		}
+	}
+}
+`)
+	)
+	require.NoError(t, New().EvalBytes(b, &doc, nil))
+	require.Len(t, doc.Foo, 1)
+
+	extra, ok := doc.Foo[0].Extra.Resource("extra_bar")
+	require.True(t, ok)
+	var e ExtraBar
+	require.NoError(t, extra.As(&e))
+	require.Equal(t, &ExtraBar{
+		Blk1: &Block1{
+			Name: "c1",
+			Type: "c1",
+		},
+		Blk2: &Block2{
+			Name: "p1",
+			Refs: []*Ref{{V: "$blk1.c1"}},
+		},
+	}, &e)
+}
+
+func Test_ScopeContextOverride(t *testing.T) {
+	type (
+		Foo struct {
+			Name string `spec:",name"`
+		}
+		Bar struct {
+			Name string `spec:",name"`
+			Attr string `spec:"attr"`
+			Ref  *Ref   `spec:"ref"`
+		}
+	)
+	var (
+		doc struct {
+			Foo []*Foo `spec:"foo"`
+			Bar []*Bar `spec:"bar"`
+		}
+		b = []byte(`
+foo "f1" {}
+bar "b1" {
+	attr = foo
+	ref  = foo.f1
+}
+`)
+	)
+	require.NoError(t, New(
+		WithScopedEnums("bar.attr", "foo"), // foo is a valid value for bar.attr.
+	).EvalBytes(b, &doc, nil))
+	require.Len(t, doc.Foo, 1)
+	require.Len(t, doc.Bar, 1)
+	require.Equal(t, &Bar{
+		Name: "b1",
+		Attr: "foo",
+		Ref:  &Ref{V: "$foo.f1"},
+	}, doc.Bar[0])
+}
