@@ -384,18 +384,31 @@ func (r *Resource) Resource(t string) (*Resource, bool) {
 
 // Attr returns the Attr by the provided name and reports whether it was found.
 func (r *Resource) Attr(name string) (*Attr, bool) {
-	return attrVal(r.Attrs, name)
+	if at, ok := attrVal(r.Attrs, name); ok {
+		return at, true
+	}
+	for _, r := range r.Children {
+		if at, ok := attrVal(r.Attrs, name); ok && r.Type == "" {
+			return at, true // Match on embedded resource.
+		}
+	}
+	return nil, false
 }
 
 // SetAttr sets the Attr on the Resource. If r is nil, a zero value Resource
 // is initialized. If an Attr with the same key exists, it is replaced by attr.
 func (r *Resource) SetAttr(attr *Attr) {
-	if r == nil {
-		*r = Resource{}
-	}
 	r.Attrs = replaceOrAppendAttr(r.Attrs, attr)
 }
 
+// EmbedAttr is like SetAttr but appends the attribute to an embedded
+// resource, cause it to be marshaled after current blocks and attributes.
+func (r *Resource) EmbedAttr(attr *Attr) {
+	r.Children = append(r.Children, &Resource{
+		Attrs: []*Attr{attr},
+	})
+}
+
 // MarshalSpec implements Marshaler.
 func (f MarshalerFunc) MarshalSpec(v any) ([]byte, error) {
 	return f(v)
