@@ -30,24 +30,37 @@ type (
 		Funcs        []*sqlspec.Func
 		Procs        []*sqlspec.Func
 		Materialized []*sqlspec.View
+		// Collected triggers to convert into spec.
+		Triggers []*schema.Trigger
 	}
-	doc struct {
-		Tables       []*sqlspec.Table  `spec:"table"`
-		Views        []*sqlspec.View   `spec:"view"`
-		Materialized []*sqlspec.View   `spec:"materialized"`
-		Funcs        []*sqlspec.Func   `spec:"function"`
-		Procs        []*sqlspec.Func   `spec:"procedure"`
-		Schemas      []*sqlspec.Schema `spec:"schema"`
+	// RealmFuncs represents the functions that used
+	// to convert the schema.Realm into HCL spec document.
+	RealmFuncs struct {
+		Schema   func(*schema.Schema) (*SchemaSpec, error)
+		Triggers func([]*schema.Trigger, *Doc) ([]*sqlspec.Trigger, error)
+	}
+	// Doc represents the common HCL spec document.
+	Doc struct {
+		Tables       []*sqlspec.Table   `spec:"table"`
+		Views        []*sqlspec.View    `spec:"view"`
+		Materialized []*sqlspec.View    `spec:"materialized"`
+		Funcs        []*sqlspec.Func    `spec:"function"`
+		Procs        []*sqlspec.Func    `spec:"procedure"`
+		Triggers     []*sqlspec.Trigger `spec:"trigger"`
+		Schemas      []*sqlspec.Schema  `spec:"schema"`
 	}
 )
 
 // Marshal marshals v into an Atlas DDL document using a schemahcl.Marshaler. Marshal uses the given
 // schemaSpec function to convert a *schema.Schema into *sqlspec.Schema, []*sqlspec.Table and []*sqlspec.View.
-func Marshal(v any, marshaler schemahcl.Marshaler, convertFunc func(*schema.Schema) (*SchemaSpec, error)) ([]byte, error) {
-	d := &doc{}
+func Marshal(v any, marshaler schemahcl.Marshaler, funcs RealmFuncs) ([]byte, error) {
+	var (
+		d  = &Doc{}
+		ts []*schema.Trigger
+	)
 	switch s := v.(type) {
 	case *schema.Schema:
-		spec, err := convertFunc(s)
+		spec, err := funcs.Schema(s)
 		if err != nil {
 			return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 		}
@@ -57,18 +70,20 @@ func Marshal(v any, marshaler schemahcl.Marshaler, convertFunc func(*schema.Sche
 		d.Schemas = []*sqlspec.Schema{spec.Schema}
 		d.Funcs = spec.Funcs
 		d.Procs = spec.Procs
+		ts = spec.Triggers
 	case *schema.Realm:
 		for _, s := range s.Schemas {
-			spec, err := convertFunc(s)
+			spec, err := funcs.Schema(s)
 			if err != nil {
 				return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 			}
 			d.Tables = append(d.Tables, spec.Tables...)
 			d.Views = append(d.Views, spec.Views...)
-			d.Materialized = spec.Materialized
+			d.Materialized = append(d.Materialized, spec.Materialized...)
 			d.Schemas = append(d.Schemas, spec.Schema)
 			d.Funcs = append(d.Funcs, spec.Funcs...)
 			d.Procs = append(d.Procs, spec.Procs...)
+			ts = append(ts, spec.Triggers...)
 		}
 		if err := QualifyObjects(d.Tables); err != nil {
 			return nil, err
@@ -91,6 +106,13 @@ func Marshal(v any, marshaler schemahcl.Marshaler, convertFunc func(*schema.Sche
 	default:
 		return nil, fmt.Errorf("specutil: failed marshaling spec. %T is not supported", v)
 	}
+	if funcs.Triggers != nil {
+		specs, err := funcs.Triggers(ts, d)
+		if err != nil {
+			return nil, err
+		}
+		d.Triggers = specs
+	}
 	return marshaler.MarshalSpec(d)
 }
 
@@ -107,28 +129,35 @@ type SchemaObject interface {
 // name in any objects with duplicate names in the provided specs.
 func QualifyObjects[T SchemaObject](specs []T) error {
 	var (
-		seen    = make(map[string]T, len(specs))
 		schemas = make(map[string]bool, len(specs))
+		byLabel = make(map[string]map[string][]T, len(specs))
 	)
 	// Loop first and qualify schema objects with the same label.
 	// For example, two tables named "users" reside in different
 	// schemas are converted to: ("s1", "users") and ("s2", "users").
 	for _, v := range specs {
-		if s, ok := seen[v.Label()]; ok {
-			schemaName, err := SchemaName(s.SchemaRef())
-			if err != nil {
-				return err
+		l := v.Label()
+		q, err := SchemaName(v.SchemaRef())
+		if err != nil {
+			return err
+		}
+		if _, ok := byLabel[l]; !ok {
+			byLabel[l] = make(map[string][]T)
+		}
+		byLabel[l][q] = append(byLabel[l][q], v)
+	}
+	for _, v := range byLabel {
+		// Multiple objects with the same label on the same
+		// schema are not qualified (repeatable blocks).
+		if len(v) == 1 {
+			continue
+		}
+		for q, sv := range v {
+			for _, s := range sv {
+				s.SetQualifier(q)
+				schemas[q] = true
 			}
-			s.SetQualifier(schemaName)
-			schemas[schemaName] = true
-			schemaName, err = SchemaName(v.SchemaRef())
-			if err != nil {
-				return err
-			}
-			v.SetQualifier(schemaName)
-			schemas[schemaName] = true
 		}
-		seen[v.Label()] = v
 	}
 	// After objects were qualified, they might be conflicted with different
 	// resources that labeled with the schema name. e.g., ("s1", "users") and
@@ -145,6 +174,32 @@ func QualifyObjects[T SchemaObject](specs []T) error {
 	return nil
 }
 
+// ColumnRefFinder returns a function that finds a column reference by its table and its schema.
+func ColumnRefFinder(specs []*sqlspec.Table) func(s, t, c string) (*schemahcl.Ref, error) {
+	var refs map[struct{ s, t string }]*sqlspec.Table
+	return func(s, t, c string) (*schemahcl.Ref, error) {
+		// Lazily initialize refs.
+		if refs == nil {
+			refs = make(map[struct{ s, t string }]*sqlspec.Table, len(specs))
+			for _, st := range specs {
+				ns, err := SchemaName(st.Schema)
+				if err != nil {
+					return nil, err
+				}
+				refs[struct{ s, t string }{s: ns, t: st.Name}] = st
+			}
+		}
+		r, ok := refs[struct{ s, t string }{s: s, t: t}]
+		if !ok {
+			return nil, fmt.Errorf("table %q.%q was not found", s, t)
+		}
+		if r.Qualifier != "" {
+			return QualifiedExternalColRef(c, r.Name, r.Qualifier), nil
+		}
+		return ExternalColumnRef(c, r.Name), nil
+	}
+}
+
 // QualifyReferences qualifies any reference with qualifier.
 func QualifyReferences(tableSpecs []*sqlspec.Table, realm *schema.Realm) error {
 	type cref struct{ s, t string }
@@ -176,9 +231,9 @@ func QualifyReferences(tableSpecs []*sqlspec.Table, realm *schema.Realm) error {
 			}
 			for i, c := range fk.RefColumns {
 				if r, ok := byRef[cref{s: fk1.RefTable.Schema.Name, t: fk1.RefTable.Name}]; ok && r.Qualifier != "" {
-					fk.RefColumns[i] = qualifiedExternalColRef(fk1.RefColumns[i].Name, r.Name, r.Qualifier)
+					fk.RefColumns[i] = QualifiedExternalColRef(fk1.RefColumns[i].Name, r.Name, r.Qualifier)
 				} else if r, ok := byRef[cref{t: fk1.RefTable.Name}]; ok && r.Qualifier == "" {
-					fk.RefColumns[i] = externalColRef(fk1.RefColumns[i].Name, r.Name)
+					fk.RefColumns[i] = ExternalColumnRef(fk1.RefColumns[i].Name, r.Name)
 				} else {
 					return fmt.Errorf("missing reference for column %q in %q.%q.%q", c.V, sname, t.Name, fk.Symbol)
 				}
