@@ -153,3 +153,77 @@ func TestCheckChangesScope(t *testing.T) {
 	})
 	require.EqualError(t, err, "found 2 schemas when migration plan is scoped to one: [\"s1\" \"s2\"]")
 }
+
+func TestBadTypeComparison(t *testing.T) {
+	var o1 struct {
+		schema.Type
+		schema.Object
+		_ []func()
+	}
+	require.Panics(t, func() { _ = schema.Type(o1) == schema.Type(o1) })
+	SortChanges([]schema.Change{
+		&schema.DropObject{O: o1},
+		&schema.DropTable{T: schema.NewTable("t1").AddColumns(schema.NewColumn("c1").SetType(o1))},
+	})
+}
+
+func TestSortChanges(t *testing.T) {
+	t1 := &schema.Table{
+		Name: "t1",
+		Columns: []*schema.Column{
+			{Name: "id", Type: &schema.ColumnType{Raw: "int"}},
+			{Name: "cid", Type: &schema.ColumnType{Raw: "int"}},
+		},
+	}
+	t1.AddForeignKeys(schema.NewForeignKey("t1").AddColumns(t1.Columns[1:2]...).SetRefTable(t1).AddRefColumns(t1.Columns[:1]...))
+	f1 := &schema.Func{Name: "f1", Deps: []schema.Object{t1}}
+	tr1 := &schema.Trigger{Name: "tr", Table: t1, Deps: []schema.Object{f1}}
+	changes := []schema.Change{
+		&schema.ModifyTable{T: t1},
+		&schema.AddTrigger{T: tr1},
+		&schema.AddTable{T: t1},
+		&schema.AddFunc{F: f1},
+	}
+	planned := SortChanges(changes)
+	require.Equal(t, []schema.Change{changes[2], changes[0], changes[3], changes[1]}, planned)
+
+	changes = []schema.Change{
+		&schema.DropTable{T: t1},
+		&schema.DropFunc{F: f1},
+		&schema.DropTrigger{T: tr1},
+	}
+	planned = SortChanges(changes)
+	require.Equal(t, []schema.Change{changes[2], changes[1], changes[0]}, planned)
+
+	// No changes.
+	planned = SortChanges([]schema.Change{
+		&schema.DropFunc{F: f1},
+		&schema.DropTable{T: t1},
+	})
+	require.Equal(t, []schema.Change{&schema.DropFunc{F: f1}, &schema.DropTable{T: t1}}, planned)
+
+	// The table must be dropped before the function if one of its triggers depends on the function.
+	t1.Triggers = []*schema.Trigger{tr1}
+	planned = SortChanges([]schema.Change{
+		&schema.DropFunc{F: f1},
+		&schema.DropTable{T: t1},
+	})
+	require.Equal(t, []schema.Change{&schema.DropTable{T: t1}, &schema.DropFunc{F: f1}}, planned)
+
+	// Ignore functions that do not reside on the same schema.
+	f1.Schema = schema.New("s1")
+	f2 := &schema.Func{Name: "f1", Args: []*schema.FuncArg{{Name: "a1"}}}
+	planned = SortChanges([]schema.Change{
+		&schema.AddFunc{F: f1},
+		&schema.DropFunc{F: f2},
+	})
+	require.Equal(t, []schema.Change{&schema.AddFunc{F: f1}, &schema.DropFunc{F: f2}}, planned)
+
+	// Order functions that reside on the same schema.
+	f2.Schema = schema.New("s1")
+	planned = SortChanges([]schema.Change{
+		&schema.AddFunc{F: f1},
+		&schema.DropFunc{F: f2},
+	})
+	require.Equal(t, []schema.Change{&schema.DropFunc{F: f2}, &schema.AddFunc{F: f1}}, planned)
+}
