@@ -41,6 +41,13 @@ type (
 	}
 )
 
+var _ interface {
+	migrate.Snapshoter
+	migrate.StmtScanner
+	migrate.CleanChecker
+	schema.TypeParseFormatter
+} = (*Driver)(nil)
+
 // DriverName holds the name used for registration.
 const DriverName = "postgres"
 
@@ -181,62 +188,81 @@ func (d *Driver) Snapshot(ctx context.Context) (migrate.RestoreFunc, error) {
 			return nil, err
 		}
 		if len(s.Tables) > 0 {
-			return nil, &migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in connected schema", s.Tables[0].Name)}
+			return nil, &migrate.NotCleanError{
+				State:  schema.NewRealm(s),
+				Reason: fmt.Sprintf("found table %q in connected schema", s.Tables[0].Name),
+			}
 		}
-		return func(ctx context.Context) error {
-			current, err := d.InspectSchema(ctx, s.Name, nil)
-			if err != nil {
-				return err
-			}
-			changes, err := d.SchemaDiff(current, s)
-			if err != nil {
-				return err
-			}
-			return d.ApplyChanges(ctx, withCascade(changes))
-		}, nil
+		return d.SchemaRestoreFunc(s), nil
 	}
 	// Not bound to a schema.
 	realm, err := d.InspectRealm(ctx, nil)
 	if err != nil {
 		return nil, err
 	}
-	restore := func(ctx context.Context) error {
-		current, err := d.InspectRealm(ctx, nil)
-		if err != nil {
-			return err
-		}
-		changes, err := d.RealmDiff(current, realm)
-		if err != nil {
-			return err
-		}
-		return d.ApplyChanges(ctx, withCascade(changes))
-	}
+	restore := d.RealmRestoreFunc(realm)
 	// Postgres is considered clean, if there are no schemas or the public schema has no tables.
 	if len(realm.Schemas) == 0 {
 		return restore, nil
 	}
 	if s, ok := realm.Schema("public"); len(realm.Schemas) == 1 && ok {
 		if len(s.Tables) > 0 {
-			return nil, &migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
+			return nil, &migrate.NotCleanError{
+				State:  realm,
+				Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name),
+			}
 		}
 		return restore, nil
 	}
-	return nil, &migrate.NotCleanError{Reason: fmt.Sprintf("found schema %q", realm.Schemas[0].Name)}
+	return nil, &migrate.NotCleanError{
+		State:  realm,
+		Reason: fmt.Sprintf("found schema %q", realm.Schemas[0].Name),
+	}
+}
+
+// SchemaRestoreFunc returns a function that restores the given schema to its desired state.
+func (d *Driver) SchemaRestoreFunc(desired *schema.Schema) migrate.RestoreFunc {
+	return func(ctx context.Context) error {
+		current, err := d.InspectSchema(ctx, desired.Name, nil)
+		if err != nil {
+			return err
+		}
+		changes, err := d.SchemaDiff(current, desired)
+		if err != nil {
+			return err
+		}
+		return d.ApplyChanges(ctx, withCascade(changes))
+	}
+}
+
+// RealmRestoreFunc returns a function that restores the given realm to its desired state.
+func (d *Driver) RealmRestoreFunc(desired *schema.Realm) migrate.RestoreFunc {
+	return func(ctx context.Context) error {
+		current, err := d.InspectRealm(ctx, nil)
+		if err != nil {
+			return err
+		}
+		changes, err := d.RealmDiff(current, desired)
+		if err != nil {
+			return err
+		}
+		return d.ApplyChanges(ctx, withCascade(changes))
+	}
 }
 
 func withCascade(changes schema.Changes) schema.Changes {
 	for _, c := range changes {
-		if d, ok := c.(*schema.DropTable); ok {
-			d.Extra = append(d.Extra, &Cascade{})
-		}
-		if d, ok := c.(*schema.DropView); ok {
-			d.Extra = append(d.Extra, &schema.IfExists{})
-		}
-		if d, ok := c.(*schema.DropProc); ok {
-			d.Extra = append(d.Extra, &schema.IfExists{})
-		}
-		if d, ok := c.(*schema.DropFunc); ok {
-			d.Extra = append(d.Extra, &schema.IfExists{})
+		switch c := c.(type) {
+		case *schema.DropTable:
+			c.Extra = append(c.Extra, &schema.IfExists{}, &Cascade{})
+		case *schema.DropView:
+			c.Extra = append(c.Extra, &schema.IfExists{}, &Cascade{})
+		case *schema.DropProc:
+			c.Extra = append(c.Extra, &schema.IfExists{}, &Cascade{})
+		case *schema.DropFunc:
+			c.Extra = append(c.Extra, &schema.IfExists{}, &Cascade{})
+		case *schema.DropObject:
+			c.Extra = append(c.Extra, &schema.IfExists{}, &Cascade{})
 		}
 	}
 	return changes
@@ -254,7 +280,7 @@ func (d *Driver) CheckClean(ctx context.Context, revT *migrate.TableIdent) error
 		case len(s.Tables) == 0, (revT.Schema == "" || s.Name == revT.Schema) && len(s.Tables) == 1 && s.Tables[0].Name == revT.Name:
 			return nil
 		default:
-			return &migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
+			return &migrate.NotCleanError{State: schema.NewRealm(s), Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
 		}
 	}
 	r, err := d.InspectRealm(ctx, nil)
@@ -265,11 +291,11 @@ func (d *Driver) CheckClean(ctx context.Context, revT *migrate.TableIdent) error
 		switch {
 		case len(s.Tables) == 0 && s.Name == "public":
 		case len(s.Tables) == 0 || s.Name != revT.Schema:
-			return &migrate.NotCleanError{Reason: fmt.Sprintf("found schema %q", s.Name)}
+			return &migrate.NotCleanError{State: r, Reason: fmt.Sprintf("found schema %q", s.Name)}
 		case len(s.Tables) > 1:
-			return &migrate.NotCleanError{Reason: fmt.Sprintf("found %d tables in schema %q", len(s.Tables), s.Name)}
+			return &migrate.NotCleanError{State: r, Reason: fmt.Sprintf("found %d tables in schema %q", len(s.Tables), s.Name)}
 		case len(s.Tables) == 1 && s.Tables[0].Name != revT.Name:
-			return &migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
+			return &migrate.NotCleanError{State: r, Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
 		}
 	}
 	return nil
@@ -280,6 +306,27 @@ func (d *Driver) Version() string {
 	return strconv.Itoa(d.conn.version)
 }
 
+// FormatType converts schema type to its column form in the database.
+func (*Driver) FormatType(t schema.Type) (string, error) {
+	return FormatType(t)
+}
+
+// ParseType returns the schema.Type value represented by the given string.
+func (*Driver) ParseType(s string) (schema.Type, error) {
+	return ParseType(s)
+}
+
+// ScanStmts implements migrate.StmtScanner.
+func (*Driver) ScanStmts(input string) ([]*migrate.Stmt, error) {
+	return (&migrate.Scanner{
+		ScannerOptions: migrate.ScannerOptions{
+			MatchBegin:       true,
+			MatchBeginAtomic: true,
+			MatchDollarQuote: true,
+		},
+	}).Scan(input)
+}
+
 func acquire(ctx context.Context, conn schema.ExecQuerier, id uint32, timeout time.Duration) error {
 	switch {
 	// With timeout (context-based).
