@@ -5,17 +5,18 @@
 package postgres
 
 import (
-	"ariga.io/atlas/schemahcl"
 	"bytes"
 	"context"
 	"database/sql"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"regexp"
 	"strconv"
 	"strings"
 	"sync"
 
+	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/internal/sqlx"
 	"ariga.io/atlas/sql/postgres/internal/postgresop"
 	"ariga.io/atlas/sql/schema"
@@ -35,9 +36,11 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 	if opts == nil {
 		opts = &schema.InspectRealmOption{}
 	}
-	r := schema.NewRealm(schemas...)
+	var (
+		r    = schema.NewRealm(schemas...)
+		mode = sqlx.ModeInspectRealm(opts)
+	)
 	if len(schemas) > 0 {
-		mode := sqlx.ModeInspectRealm(opts)
 		if mode.Is(schema.InspectTables) {
 			if err := i.inspectTables(ctx, r, nil); err != nil {
 				return nil, err
@@ -62,8 +65,21 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 				return nil, err
 			}
 		}
+		if mode.Is(schema.InspectObjects) {
+			if err := i.inspectSequences(ctx, r, nil); err != nil {
+				return nil, err
+			}
+		}
+		if mode.Is(schema.InspectTriggers) {
+			if err := i.inspectTriggers(ctx, r, nil); err != nil {
+				return nil, err
+			}
+		}
+		if err := i.inspectDeps(ctx, r, nil); err != nil {
+			return nil, err
+		}
 	}
-	return sqlx.ExcludeRealm(r, opts.Exclude)
+	return schema.ExcludeRealm(r, opts.Exclude)
 }
 
 // InspectSchema returns schema descriptions of the tables in the given schema.
@@ -75,6 +91,10 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 	}
 	switch n := len(schemas); {
 	case n == 0:
+		// Empty string indicates current connected schema.
+		if name == "" {
+			return nil, &schema.NotExistError{Err: errors.New("postgres: current_schema() defined in search_path was not found")}
+		}
 		return nil, &schema.NotExistError{Err: fmt.Errorf("postgres: schema %q was not found", name)}
 	case n > 1:
 		return nil, fmt.Errorf("postgres: %d schemas were found for %q", n, name)
@@ -82,8 +102,10 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 	if opts == nil {
 		opts = &schema.InspectOptions{}
 	}
-	r := schema.NewRealm(schemas...)
-	mode := sqlx.ModeInspectSchema(opts)
+	var (
+		r    = schema.NewRealm(schemas...)
+		mode = sqlx.ModeInspectSchema(opts)
+	)
 	if mode.Is(schema.InspectTables) {
 		if err := i.inspectTables(ctx, r, opts); err != nil {
 			return nil, err
@@ -108,7 +130,20 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 			return nil, err
 		}
 	}
-	return sqlx.ExcludeSchema(r.Schemas[0], opts.Exclude)
+	if mode.Is(schema.InspectObjects) {
+		if err := i.inspectSequences(ctx, r, opts); err != nil {
+			return nil, err
+		}
+	}
+	if mode.Is(schema.InspectTriggers) {
+		if err := i.inspectTriggers(ctx, r, nil); err != nil {
+			return nil, err
+		}
+	}
+	if err := i.inspectDeps(ctx, r, opts); err != nil {
+		return nil, err
+	}
+	return schema.ExcludeSchema(r.Schemas[0], opts.Exclude)
 }
 
 func (i *inspect) inspectTables(ctx context.Context, r *schema.Realm, opts *schema.InspectOptions) error {
@@ -159,8 +194,11 @@ func (i *inspect) tables(ctx context.Context, realm *schema.Realm, opts *schema.
 	}
 	defer rows.Close()
 	for rows.Next() {
-		var tSchema, name, comment, partattrs, partstart, partexprs sql.NullString
-		if err := rows.Scan(&tSchema, &name, &comment, &partattrs, &partstart, &partexprs); err != nil {
+		var (
+			oid                                                     sql.NullInt64
+			tSchema, name, comment, partattrs, partstart, partexprs sql.NullString
+		)
+		if err := rows.Scan(&oid, &tSchema, &name, &comment, &partattrs, &partstart, &partexprs); err != nil {
 			return fmt.Errorf("scan table information: %w", err)
 		}
 		if !sqlx.ValidString(tSchema) || !sqlx.ValidString(name) {
@@ -172,6 +210,9 @@ func (i *inspect) tables(ctx context.Context, realm *schema.Realm, opts *schema.
 		}
 		t := schema.NewTable(name.String)
 		s.AddTables(t)
+		if oid.Valid {
+			t.AddAttrs(&OID{V: oid.Int64})
+		}
 		if sqlx.ValidString(comment) {
 			t.SetComment(comment.String)
 		}
@@ -776,6 +817,12 @@ type (
 		T string // e.g., void, any, cstring, etc.
 	}
 
+	// OID is the object identifier as defined in the Postgres catalog.
+	OID struct {
+		schema.Attr
+		V int64
+	}
+
 	// enumType represents an enum type. It serves aa intermediate representation of a Postgres enum type,
 	// to temporary save TypeID and TypeName of an enum column until the enum values can be extracted.
 	enumType struct {
@@ -811,6 +858,8 @@ type (
 		Null    bool            // Nullability.
 		Default schema.Expr     // Default value.
 		Checks  []*schema.Check // Check constraints.
+		Attrs   []schema.Attr   // Extra attributes, such as OID.
+		Deps    []schema.Object // Objects this domain depends on.
 	}
 
 	// IntervalType defines an interval type.
@@ -889,10 +938,27 @@ type (
 	// Sequence defines (the supported) sequence options.
 	// https://postgresql.org/docs/current/sql-createsequence.html
 	Sequence struct {
-		Start, Increment int64
+		schema.Object
+		// Fields used by the Identity schema attribute.
+		Start     int64
+		Increment int64
 		// Last sequence value written to disk.
 		// https://postgresql.org/docs/current/view-pg-sequences.html.
 		Last int64
+
+		// Field used when defining and managing independent
+		// sequences (not part of IDENTITY or serial columns).
+		Name     string         // Sequence name.
+		Schema   *schema.Schema // Optional schema.
+		Type     schema.Type    // Sequence type.
+		Cache    int64          // Cache size.
+		Min, Max *int64         // Min and max values.
+		Cycle    bool           // Whether the sequence cycles.
+		Attrs    []schema.Attr  // Additional attributes (e.g., comments),
+		Owner    struct {       // Optional owner of the sequence.
+			T *schema.Table
+			C *schema.Column
+		}
 	}
 
 	// Identity defines an identity column.
@@ -1018,6 +1084,16 @@ func (d *DomainType) Ref() *schemahcl.Ref {
 	return &schemahcl.Ref{V: "$domain." + d.T}
 }
 
+// Underlying returns the underlying type of the domain.
+func (d *DomainType) Underlying() schema.Type {
+	return d.Type
+}
+
+// Underlying returns the underlying type of the array.
+func (a *ArrayType) Underlying() schema.Type {
+	return a.Type
+}
+
 // String implements fmt.Stringer interface.
 func (o ReferenceOption) String() string {
 	return string(o)
@@ -1275,6 +1351,7 @@ ORDER BY
 	// Query to list table information.
 	tablesQuery = `
 SELECT
+	t3.oid,
 	t1.table_schema,
 	t1.table_name,
 	pg_catalog.obj_description(t3.oid, 'pg_class') AS comment,
@@ -1297,6 +1374,7 @@ ORDER BY
 `
 	tablesQueryArgs = `
 SELECT
+	t3.oid,
 	t1.table_schema,
 	t1.table_name,
 	pg_catalog.obj_description(t3.oid, 'pg_class') AS comment,
