@@ -23,18 +23,20 @@ import (
 
 type (
 	doc struct {
-		Tables       []*sqlspec.Table  `spec:"table"`
-		Views        []*sqlspec.View   `spec:"view"`
-		Materialized []*sqlspec.View   `spec:"materialized"`
-		Enums        []*Enum           `spec:"enum"`
-		Domains      []*Domain         `spec:"domain"`
-		Funcs        []*sqlspec.Func   `spec:"function"`
-		Procs        []*sqlspec.Func   `spec:"procedure"`
-		Schemas      []*sqlspec.Schema `spec:"schema"`
+		Tables       []*sqlspec.Table   `spec:"table"`
+		Views        []*sqlspec.View    `spec:"view"`
+		Materialized []*sqlspec.View    `spec:"materialized"`
+		Enums        []*enum            `spec:"enum"`
+		Domains      []*domain          `spec:"domain"`
+		Sequences    []*sequence        `spec:"sequence"`
+		Funcs        []*sqlspec.Func    `spec:"function"`
+		Procs        []*sqlspec.Func    `spec:"procedure"`
+		Triggers     []*sqlspec.Trigger `spec:"trigger"`
+		Schemas      []*sqlspec.Schema  `spec:"schema"`
 	}
 
 	// Enum holds a specification for an enum type.
-	Enum struct {
+	enum struct {
 		Name      string         `spec:",name"`
 		Qualifier string         `spec:",qualifier"`
 		Schema    *schemahcl.Ref `spec:"schema"`
@@ -43,7 +45,7 @@ type (
 	}
 
 	// Domain holds a specification for a domain type.
-	Domain struct {
+	domain struct {
 		Name      string           `spec:",name"`
 		Qualifier string           `spec:",qualifier"`
 		Schema    *schemahcl.Ref   `spec:"schema"`
@@ -53,47 +55,84 @@ type (
 		Checks    []*sqlspec.Check `spec:"check"`
 		schemahcl.DefaultExtension
 	}
+
+	// Sequence holds a specification for a sequence.
+	sequence struct {
+		Name      string         `spec:",name"`
+		Qualifier string         `spec:",qualifier"`
+		Schema    *schemahcl.Ref `spec:"schema"`
+		// Type, Start, Increment, Min, Max, Cache, Cycle
+		// are optionally added to the sequence definition.
+		schemahcl.DefaultExtension
+	}
 )
 
 // merge merges the doc d1 into d.
 func (d *doc) merge(d1 *doc) {
-	d.Tables = append(d.Tables, d1.Tables...)
-	d.Views = append(d.Views, d1.Views...)
-	d.Materialized = append(d.Materialized, d1.Materialized...)
-	d.Domains = append(d.Domains, d1.Domains...)
 	d.Enums = append(d.Enums, d1.Enums...)
 	d.Funcs = append(d.Funcs, d1.Funcs...)
 	d.Procs = append(d.Procs, d1.Procs...)
+	d.Views = append(d.Views, d1.Views...)
+	d.Tables = append(d.Tables, d1.Tables...)
+	d.Domains = append(d.Domains, d1.Domains...)
 	d.Schemas = append(d.Schemas, d1.Schemas...)
+	d.Sequences = append(d.Sequences, d1.Sequences...)
+	d.Triggers = append(d.Triggers, d1.Triggers...)
+	d.Materialized = append(d.Materialized, d1.Materialized...)
+}
+
+func (d *doc) ScanDoc() *specutil.ScanDoc {
+	return &specutil.ScanDoc{
+		Schemas:      d.Schemas,
+		Tables:       d.Tables,
+		Views:        d.Views,
+		Funcs:        d.Funcs,
+		Procs:        d.Procs,
+		Triggers:     d.Triggers,
+		Materialized: d.Materialized,
+	}
 }
 
 // Label returns the defaults label used for the enum resource.
-func (e *Enum) Label() string { return e.Name }
+func (e *enum) Label() string { return e.Name }
 
 // QualifierLabel returns the qualifier label used for the enum resource, if any.
-func (e *Enum) QualifierLabel() string { return e.Qualifier }
+func (e *enum) QualifierLabel() string { return e.Qualifier }
 
 // SetQualifier sets the qualifier label used for the enum resource.
-func (e *Enum) SetQualifier(q string) { e.Qualifier = q }
+func (e *enum) SetQualifier(q string) { e.Qualifier = q }
 
 // SchemaRef returns the schema reference for the enum.
-func (e *Enum) SchemaRef() *schemahcl.Ref { return e.Schema }
+func (e *enum) SchemaRef() *schemahcl.Ref { return e.Schema }
 
-// Label returns the defaults label used for the enum resource.
-func (d *Domain) Label() string { return d.Name }
+// Label returns the defaults label used for the domain resource.
+func (d *domain) Label() string { return d.Name }
 
-// QualifierLabel returns the qualifier label used for the enum resource, if any.
-func (d *Domain) QualifierLabel() string { return d.Qualifier }
+// QualifierLabel returns the qualifier label used for the domain resource, if any.
+func (d *domain) QualifierLabel() string { return d.Qualifier }
 
-// SetQualifier sets the qualifier label used for the enum resource.
-func (d *Domain) SetQualifier(q string) { d.Qualifier = q }
+// SetQualifier sets the qualifier label used for the domain resource.
+func (d *domain) SetQualifier(q string) { d.Qualifier = q }
 
-// SchemaRef returns the schema reference for the enum.
-func (d *Domain) SchemaRef() *schemahcl.Ref { return d.Schema }
+// SchemaRef returns the schema reference for the domain.
+func (d *domain) SchemaRef() *schemahcl.Ref { return d.Schema }
+
+// Label returns the defaults label used for the sequence resource.
+func (s *sequence) Label() string { return s.Name }
+
+// QualifierLabel returns the qualifier label used for the sequence resource, if any.
+func (s *sequence) QualifierLabel() string { return s.Qualifier }
+
+// SetQualifier sets the qualifier label used for the sequence resource.
+func (s *sequence) SetQualifier(q string) { s.Qualifier = q }
+
+// SchemaRef returns the schema reference for the sequence.
+func (s *sequence) SchemaRef() *schemahcl.Ref { return s.Schema }
 
 func init() {
-	schemahcl.Register("enum", &Enum{})
-	schemahcl.Register("domain", &Domain{})
+	schemahcl.Register("enum", &enum{})
+	schemahcl.Register("domain", &domain{})
+	schemahcl.Register("sequence", &sequence{})
 }
 
 // evalSpec evaluates an Atlas DDL document into v using the input.
@@ -104,10 +143,7 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := hclState.Eval(p, &d, input); err != nil {
 			return err
 		}
-		if err := specutil.Scan(v,
-			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views, Materialized: d.Materialized, Funcs: d.Funcs, Procs: d.Procs},
-			scanFuncs,
-		); err != nil {
+		if err := specutil.Scan(v, d.ScanDoc(), scanFuncs); err != nil {
 			return fmt.Errorf("specutil: failed converting to *schema.Realm: %w", err)
 		}
 		if err := convertEnums(d.Tables, d.Enums, v); err != nil {
@@ -116,6 +152,9 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := convertDomains(d.Tables, d.Domains, v); err != nil {
 			return err
 		}
+		if err := convertSequences(d.Tables, d.Sequences, v); err != nil {
+			return err
+		}
 	case *schema.Schema:
 		var d doc
 		if err := hclState.Eval(p, &d, input); err != nil {
@@ -125,10 +164,7 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 			return fmt.Errorf("specutil: expecting document to contain a single schema, got %d", len(d.Schemas))
 		}
 		r := &schema.Realm{}
-		if err := specutil.Scan(r,
-			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views, Materialized: d.Materialized, Funcs: d.Funcs, Procs: d.Procs},
-			scanFuncs,
-		); err != nil {
+		if err := specutil.Scan(r, d.ScanDoc(), scanFuncs); err != nil {
 			return err
 		}
 		if err := convertEnums(d.Tables, d.Enums, r); err != nil {
@@ -137,6 +173,9 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := convertDomains(d.Tables, d.Domains, r); err != nil {
 			return err
 		}
+		if err := convertSequences(d.Tables, d.Sequences, r); err != nil {
+			return err
+		}
 		*v = *r.Schemas[0]
 	case schema.Schema, schema.Realm:
 		return fmt.Errorf("postgres: Eval expects a pointer: received %[1]T, expected *%[1]T", v)
@@ -148,21 +187,26 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 
 // MarshalSpec marshals v into an Atlas DDL document using a schemahcl.Marshaler.
 func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
-	var d doc
+	var (
+		d  doc
+		ts []*schema.Trigger
+	)
 	switch s := v.(type) {
 	case *schema.Schema:
-		d1, err := schemaSpec(s)
+		d1, trs, err := schemaSpec(s)
 		if err != nil {
 			return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 		}
+		ts = trs
 		d.merge(d1)
 	case *schema.Realm:
 		for _, s := range s.Schemas {
-			d1, err := schemaSpec(s)
+			d1, trs, err := schemaSpec(s)
 			if err != nil {
 				return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 			}
 			d.merge(d1)
+			ts = append(ts, trs...)
 		}
 		if err := specutil.QualifyObjects(d.Tables); err != nil {
 			return nil, err
@@ -179,6 +223,9 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 		if err := specutil.QualifyObjects(d.Domains); err != nil {
 			return nil, err
 		}
+		if err := specutil.QualifyObjects(d.Sequences); err != nil {
+			return nil, err
+		}
 		if err := specutil.QualifyObjects(d.Funcs); err != nil {
 			return nil, err
 		}
@@ -188,9 +235,15 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 		if err := specutil.QualifyReferences(d.Tables, s); err != nil {
 			return nil, err
 		}
+		if err := qualifySeqRefs(d.Sequences, d.Tables, s); err != nil {
+			return nil, err
+		}
 	default:
 		return nil, fmt.Errorf("specutil: failed marshaling spec. %T is not supported", v)
 	}
+	if err := triggersSpec(ts, &d); err != nil {
+		return nil, err
+	}
 	return marshaler.MarshalSpec(&d)
 }
 
@@ -534,7 +587,7 @@ func convertColumnType(spec *sqlspec.Column) (schema.Type, error) {
 
 // convertEnums converts possibly referenced column types (like enums) to
 // an actual schema.Type and sets it on the correct schema.Column.
-func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error {
+func convertEnums(tables []*sqlspec.Table, enums []*enum, r *schema.Realm) error {
 	if len(enums) == 0 {
 		return nil
 	}
@@ -621,10 +674,10 @@ func enumRef(n string) *schemahcl.Ref {
 }
 
 // schemaSpec converts from a concrete Postgres schema to Atlas specification.
-func schemaSpec(s *schema.Schema) (*doc, error) {
+func schemaSpec(s *schema.Schema) (*doc, []*schema.Trigger, error) {
 	spec, err := specutil.FromSchema(s, specFuncs)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	d := &doc{
 		Tables:       spec.Tables,
@@ -633,13 +686,13 @@ func schemaSpec(s *schema.Schema) (*doc, error) {
 		Funcs:        spec.Funcs,
 		Procs:        spec.Procs,
 		Schemas:      []*sqlspec.Schema{spec.Schema},
-		Enums:        make([]*Enum, 0, len(s.Objects)),
-		Domains:      make([]*Domain, 0, len(s.Objects)),
+		Enums:        make([]*enum, 0, len(s.Objects)),
+		Domains:      make([]*domain, 0, len(s.Objects)),
 	}
 	if err := objectSpec(d, spec, s); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
-	return d, nil
+	return d, spec.Triggers, nil
 }
 
 // tableSpec converts from a concrete Postgres sqlspec.Table to a schema.Table.
@@ -863,7 +916,6 @@ var TypeRegistry = schemahcl.NewRegistry(
 		schemahcl.NewTypeSpec(TypeDateRange),
 		schemahcl.NewTypeSpec(TypeDateMultiRange),
 		schemahcl.NewTypeSpec("hstore"),
-		schemahcl.NewTypeSpec("sql", schemahcl.WithAttributes(&schemahcl.TypeAttr{Name: "def", Required: true, Kind: reflect.String})),
 	),
 	// PostgreSQL internal, pseudo, and special types.
 	schemahcl.WithSpecs(func() (specs []*schemahcl.TypeSpec) {
