@@ -12,7 +12,6 @@ import (
 	"ariga.io/atlas/sql/internal/spectest"
 	"ariga.io/atlas/sql/internal/sqlx"
 	"ariga.io/atlas/sql/schema"
-	"github.com/hashicorp/hcl/v2/hclparse"
 	"github.com/stretchr/testify/require"
 )
 
@@ -739,7 +738,7 @@ table "logs" {
 				}
 			}
 		`), &schema.Schema{}, nil)
-		require.EqualError(t, err, `specutil: cannot convert table "logs": missing columns or expressions for logs.partition`)
+		require.EqualError(t, err, `cannot convert table "logs": missing columns or expressions for logs.partition`)
 
 		err = EvalHCLBytes([]byte(`
 			schema "test" {}
@@ -753,7 +752,7 @@ table "logs" {
 				}
 			}
 		`), &schema.Schema{}, nil)
-		require.EqualError(t, err, `specutil: cannot convert table "logs": multiple definitions for logs.partition, use "columns" or "by"`)
+		require.EqualError(t, err, `cannot convert table "logs": multiple definitions for logs.partition, use "columns" or "by"`)
 	})
 }
 
@@ -2147,6 +2146,34 @@ schema "s2" {
 		string(got))
 }
 
+func TestMarshalSkipQualifiers(t *testing.T) {
+	buf, err := MarshalHCL.MarshalSpec(
+		schema.New("s1").
+			AddTables(
+				schema.NewTable("t1").AddColumns(schema.NewIntColumn("id1", "int")),
+				schema.NewTable("t1").AddColumns(schema.NewIntColumn("id2", "int")),
+			),
+	)
+	require.NoError(t, err)
+	require.Equal(t, `table "t1" {
+  schema = schema.s1
+  column "id1" {
+    null = false
+    type = int
+  }
+}
+table "t1" {
+  schema = schema.s1
+  column "id2" {
+    null = false
+    type = int
+  }
+}
+schema "s1" {
+}
+`, string(buf), "qualifiers are skipped if objects belong to the same schema (repeatable blocks)")
+}
+
 func TestMarshalQualifiers(t *testing.T) {
 	var (
 		s1 = schema.New("s1").
@@ -2211,52 +2238,3 @@ schema "s3" {
 }
 `, string(buf))
 }
-
-func TestEvalHCLError(t *testing.T) {
-	for _, tt := range []struct {
-		text, err string
-	}{
-		{
-			text: `
-			table "test" {
-			  column "id" {
-			    type = string
-			  }
-			}`,
-			err: `schema.hcl:4,15-21: Unknown column.type; There is no type named "string".`,
-		},
-		{
-			text: `
-			table "test" {
-			  index "i" {
-			    type = unknown
-			  }
-			}`,
-			err: `schema.hcl:4,15-22: Unknown index.type; There is no type named "unknown".`,
-		},
-		{
-			text: `
-			table "test" {
-			  foreign_key "f" {
-			    on_delete = unknown
-			  }
-			}`,
-			err: `schema.hcl:4,20-27: Unknown foreign_key.on_delete; There is no on_delete named "unknown".`,
-		},
-		{
-			text: `
-			table "test" {
-			  partition {
-			    type = UNKNOWN
-			  }
-			}`,
-			err: `schema.hcl:4,15-22: Unknown partition.type; There is no type named "UNKNOWN".`,
-		},
-	} {
-		p := hclparse.NewParser()
-		_, diags := p.ParseHCL([]byte(tt.text), "schema.hcl")
-		require.False(t, diags.HasErrors())
-		err := EvalHCL.Eval(p, &schema.Schema{}, nil)
-		require.EqualError(t, err, tt.err)
-	}
-}
