@@ -39,18 +39,21 @@ type (
 		Indexes     []*Index
 		PrimaryKey  *Index
 		ForeignKeys []*ForeignKey
-		Attrs       []Attr // Attrs, constraints and options.
+		Attrs       []Attr     // Attrs, constraints and options.
+		Triggers    []*Trigger // Triggers on the table.
+		Deps        []Object   // Objects this table depends on.
 	}
 
 	// A View represents a view definition.
 	View struct {
-		Name    string
-		Def     string
-		Schema  *Schema
-		Columns []*Column
-		Attrs   []Attr   // Attrs and options.
-		Deps    []Object // Tables and views used in view definition.
-		Indexes []*Index // Indexes on materialized view.
+		Name     string
+		Def      string
+		Schema   *Schema
+		Columns  []*Column
+		Attrs    []Attr     // Attrs and options.
+		Indexes  []*Index   // Indexes on materialized view.
+		Triggers []*Trigger // Triggers on the view.
+		Deps     []Object   // Objects this view depends on.
 	}
 
 	// A Column represents a column definition.
@@ -108,15 +111,42 @@ type (
 		OnDelete   ReferenceOption
 	}
 
+	// A Trigger represents a trigger definition.
+	Trigger struct {
+		Name string
+		// Table or View that this trigger belongs to.
+		Table      *Table
+		View       *View
+		ActionTime TriggerTime    // BEFORE, AFTER, or INSTEAD OF.
+		Events     []TriggerEvent // INSERT, UPDATE, DELETE, etc.
+		For        TriggerFor     // FOR EACH ROW or FOR EACH STATEMENT.
+		Body       string         // Trigger body only.
+		Attrs      []Attr         // WHEN, REFERENCING, etc.
+		Deps       []Object       // Objects this trigger depends on.
+	}
+
+	// TriggerTime represents the trigger action time.
+	TriggerTime string
+
+	// TriggerFor represents the trigger FOR EACH spec.
+	TriggerFor string
+
+	// TriggerEvent represents the trigger event.
+	TriggerEvent struct {
+		Name    string    // Name of the event (e.g. INSERT, UPDATE, DELETE).
+		Columns []*Column // Columns that might be associated with the event.
+	}
+
 	// Func represents a function definition.
 	Func struct {
 		Name   string
 		Schema *Schema
 		Args   []*FuncArg
 		Ret    Type
-		Body   string // Function body only.
-		Lang   string // Language (e.g. SQL, PL/pgSQL, etc.).
-		Attrs  []Attr // Extra driver specific attributes.
+		Body   string   // Function body only.
+		Lang   string   // Language (e.g. SQL, PL/pgSQL, etc.).
+		Attrs  []Attr   // Extra driver specific attributes.
+		Deps   []Object // Objects this function depends on.
 	}
 
 	// Proc represents a procedure definition.
@@ -124,9 +154,10 @@ type (
 		Name   string
 		Schema *Schema
 		Args   []*FuncArg
-		Body   string // Function body only.
-		Lang   string // Language (e.g. SQL, PL/pgSQL, etc.).
-		Attrs  []Attr // Extra driver specific attributes.
+		Body   string   // Function body only.
+		Lang   string   // Language (e.g. SQL, PL/pgSQL, etc.).
+		Attrs  []Attr   // Extra driver specific attributes.
+		Deps   []Object // Objects this function depends on.
 	}
 
 	// A FuncArg represents a single function argument.
@@ -150,6 +181,32 @@ const (
 	FuncArgModeVariadic FuncArgMode = "VARIADIC"
 )
 
+// List of supported trigger action times.
+const (
+	TriggerTimeBefore  TriggerTime = "BEFORE"
+	TriggerTimeAfter   TriggerTime = "AFTER"
+	TriggerTimeInstead TriggerTime = "INSTEAD OF"
+)
+
+// List of supported trigger FOR EACH spec.
+const (
+	TriggerForRow  TriggerFor = "ROW"
+	TriggerForStmt TriggerFor = "STATEMENT"
+)
+
+// List of supported trigger events.
+var (
+	TriggerEventInsert   = TriggerEvent{Name: "INSERT"}
+	TriggerEventUpdate   = TriggerEvent{Name: "UPDATE"}
+	TriggerEventDelete   = TriggerEvent{Name: "DELETE"}
+	TriggerEventTruncate = TriggerEvent{Name: "TRUNCATE"}
+)
+
+// TriggerEventUpdateOf returns an UPDATE OF trigger event.
+func TriggerEventUpdateOf(columns ...*Column) TriggerEvent {
+	return TriggerEvent{Name: "UPDATE OF", Columns: columns}
+}
+
 // Schema returns the first schema that matched the given name.
 func (r *Realm) Schema(name string) (*Schema, bool) {
 	for _, s := range r.Schemas {
@@ -250,6 +307,16 @@ func (t *Table) ForeignKey(symbol string) (*ForeignKey, bool) {
 	return nil, false
 }
 
+// Trigger returns the first trigger that matches the given name.
+func (t *Table) Trigger(name string) (*Trigger, bool) {
+	for _, r := range t.Triggers {
+		if r.Name == name {
+			return r, true
+		}
+	}
+	return nil, false
+}
+
 // Materialized reports if the view is materialized.
 func (v *View) Materialized() bool {
 	for _, a := range v.Attrs {
@@ -290,6 +357,16 @@ func (v *View) Index(name string) (*Index, bool) {
 	return nil, false
 }
 
+// Trigger returns the first trigger that matches the given name.
+func (v *View) Trigger(name string) (*Trigger, bool) {
+	for _, r := range v.Triggers {
+		if r.Name == name {
+			return r, true
+		}
+	}
+	return nil, false
+}
+
 // AsTable returns a table that represents the view.
 func (v *View) AsTable() *Table {
 	return NewTable(v.Name).
@@ -420,6 +497,28 @@ type (
 	UnsupportedType struct {
 		T string
 	}
+
+	// TypeParser is an interface that is required be implemented by
+	// different drivers for parsing column types from their database
+	// forms to the schema representation.
+	TypeParser interface {
+		// ParseType converts the raw database type to its schema.Type representation.
+		ParseType(string) (Type, error)
+	}
+
+	// TypeFormatter is an interface that is required to be implemented by
+	// different drivers to format column types into their corresponding
+	// database forms.
+	TypeFormatter interface {
+		// FormatType converts a schema type to its column form in the database.
+		FormatType(Type) (string, error)
+	}
+
+	// TypeParseFormatter that groups the TypeParser and TypeFormatter interfaces.
+	TypeParseFormatter interface {
+		TypeParser
+		TypeFormatter
+	}
 )
 
 type (
@@ -509,6 +608,9 @@ const (
 // objects.
 func (*Table) obj()    {}
 func (*View) obj()     {}
+func (*Func) obj()     {}
+func (*Proc) obj()     {}
+func (*Trigger) obj()  {}
 func (*EnumType) obj() {}
 
 // expressions.
@@ -544,3 +646,11 @@ func UnderlyingExpr(x Expr) Expr {
 	}
 	return x
 }
+
+// UnderlyingType returns the underlying type of t.
+func UnderlyingType(t Type) Type {
+	if w, ok := t.(interface{ Underlying() Type }); ok {
+		return UnderlyingType(w.Underlying())
+	}
+	return t
+}
